package lbl.akenti.certs;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.security.GeneralSecurityException;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.Signature;
import java.text.ParseException;
import lbl.akenti.certs.elements.AkentiInputStream;
import lbl.akenti.certs.elements.AkentiIssuer;
import lbl.akenti.certs.elements.AkentiOutputStream;
import lbl.akenti.util.Base64;
import lbl.akenti.util.ParsingException;
import lbl.akenti.util.UTCTime;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * This abstract class is the superclass of all <code>Akenti</code> certificates.
 *
 * <p>
 * We have designed all the certificates to be immutable.
 * Subclass Implementors should adhere to this policy.                <BR>
 *
 *  1) The set methods should be accessed by the genarators only.     <BR>
 *
 *  2) The doReadObject should be protected.                          <BR>
 *
 *  3) The doFromXML should be protected.                             <BR>
 *
 * <p>
 * Encoding Rules for the canonical form:                             <BR>
 *   1) The various fields are separated by one blank space.          <BR>
 *
 *   2) The <CODE>AkentiOutputStream</CODE> is configured                   <BR>
 *      to write primitive types, strings and byte arrays.
 *
 * <p>
 * Note:                                                              <BR>
 *   The encoding is straightforward with the exception
 *   of the version number which is preceded by
 *   the character 'V'.
 *
 * <pre>
 * <b>CertificateEncoding</b> {
 *    type             // certificate's type
 *    version          // certificate's version number
 *    id               // certificate's unique id
 *    issuer           // issuer and signer of certificate
 *    notBefore        // start validity date
 *    notAfter         // end validity date
 *    canonAlg         // name of algorithm used to format certificate
 *                        for signing or pem encoding.
 *    signatureAlg     // name of algorithm used to sign this certificate
 *    ...              // fields contained in the concrete subclasses
 *    signature        // digital signature of the certificate
 *  }
 * </pre>
 * @see AkentiCertGenerator
 * @see AkentiCertificate
 * @see AkentiOutputStream
 * @see AkentiInputStream
 * @see AkentiIssuer
 * @see lbl.akenti.util.UTCTime
 * @version 1.1 02/04/17
 * @author Srilekha Mudumbai
 * @author Abdelilah Essiari
 */

public abstract class AkentiCertificate extends Certificate {
  protected int version;
  protected String id;
  protected AkentiIssuer issuer;
  protected UTCTime notBefore, notAfter;
  protected String canonAlg;
  protected String signatureAlg;
  protected byte[] signature;
  
  /**
   * Constructs this <CODE>AkentiCertificate</CODE>.
   * @param type certificate type
   */
  protected AkentiCertificate(String type) {
    super(type);
    version = 2;
    id = "NO_ID";
    notBefore = notAfter = new UTCTime();
    signatureAlg = "NO_SIG_ALG";
    canonAlg = "Ak1CanAlg";
    signature = new String("NO_SIG").getBytes();
  }
  
  /**
   * Constructs this object from an <CODE>AkentiInputStream</CODE>.
   */
  protected AkentiCertificate(AkentiInputStream in)
  throws IOException, ParseException {
    super("NO_TYPE");
    readObject(in);
  }
  
  /**
   * Returns the approriate hash string whose digest will be used to
   * locate certificates.
   *
   **/
  public abstract String getHashString();
  
  /**
   * Returns the version number.
   */
  public int getVersion() {
    return version;
  }
  
  /**
   * Returns this <CODE>AkentiCertificate</CODE>'s unique id.
   */
  public String getUniqueID() {
    return id;
  }
  
  /**
   * Returns this <CODE>AkentiCertificate</CODE>'s issuer.
   */
  public AkentiIssuer getIssuer() {
    return issuer;
  }
  
  /**
   * Returns the start validity date.
   */
  public UTCTime getNotBefore() {
    return notBefore;
  }
  
  /**
   * Returns the end validity date.
   */
  public UTCTime getNotAfter() {
    return notAfter;
  }
  
  /**
   * Returns the name of the signature algorithm.
   */
  public String getSignatureAlg() {
    return signatureAlg;
  }
  
  /**
   * Returns the name of the canonalization algorithm.
   */
  public String getCanonAlg() {
    return canonAlg;
  }
  
  /**
   * Returns the signature.
   */
  public byte[] getSignature() {
    return signature;
  }
  
  /**
   * Returns the signable part of this certificate.
   *
   * @see #writeObject(AkentiOutputStream)
   */
  public byte[] getSignable() throws IOException {
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    
    writeObject(new AkentiOutputStream(baos), false);
    return baos.toByteArray();
  }
  
  /**
   * Signs this certificate.
   *
   * @see getSignable
   * @exception NoSuchAlgorithmException
   * @exception InvalidKeyException
   * @exception SignatureException
   */
  final void
  sign(String signatureAlg, PrivateKey privateKey)
  throws GeneralSecurityException, IOException {
    Signature sig = Signature.getInstance(signatureAlg);
    
    sig.initSign(privateKey);
    
    this.signatureAlg = System.getProperty(signatureAlg, signatureAlg);
    sig.update(getSignable());
    signature = sig.sign();
  }
  
  /**
   * Verifies the signature.
   *
   * @exception NoSuchAlgorithmException
   * @exception InvalidKeyException
   * @exception SignatureException
   */
  public boolean verify(PublicKey publicKey)
  throws GeneralSecurityException, IOException {
    
    Signature sig = Signature.getInstance(System.getProperty(signatureAlg, signatureAlg));
    
    sig.initVerify(publicKey);
    sig.update(getSignable());
    return sig.verify(signature);
  }
  
  /**
   * Returns an instance of this certificate.
   */
  protected abstract Certificate newInstance();
  
  public final void
  writeObject(AkentiOutputStream drain) throws IOException {
    writeObject(drain, true);
  }
  
  private void
  writeObject(AkentiOutputStream drain, boolean includeSig)
  throws IOException {
    drain.write(type);
    drain.write(' ');
    drain.write('V');
    drain.write(version);
    drain.write(' ');
    drain.write(id);
    drain.write(' ');
    issuer.writeObject(drain);
    drain.write(' ');
    drain.write(notBefore.toString());
    drain.write(' ');
    drain.write(notAfter.toString());
    drain.write(' ');
    drain.write(canonAlg);
    drain.write(' ');
    drain.write(signatureAlg);
    drain.write(' ');
    doWrite(drain);
    
    if (includeSig) {
      drain.write(' ');
      drain.write(signature.length);
      drain.write(' ');
      drain.write(signature);
    }
  }
  
  protected final void
  readObject(AkentiInputStream source)
  throws IOException, ParseException {
    type = source.readString();
    
    if (source.readChar() != 'V') {
      throw new ParsingException("reading version failed");
    }
    
    version = source.readInt();
    id = source.readString();
    issuer = new AkentiIssuer(source);
    notBefore = new UTCTime(source.readString());
    notAfter = new UTCTime(source.readString());
    canonAlg = source.readString();
    signatureAlg = source.readString();
    doRead(source);
    signature = source.readBytes(source.readInt());
  }
  
  protected final void
  fromXML(Element akCert) throws ParseException {
    Node child = akCert.getFirstChild();
    
    if (child == null || child.getNodeType() != Node.ELEMENT_NODE) {
      throw new ParsingException();
    }
    
    Element signable = (Element) child;
    Node temp = signable.getFirstChild();
    
    if (temp == null || temp.getNodeType() != Node.ELEMENT_NODE) {
      throw new ParsingException();
    }
    
    Element header = (Element) temp;
    String rep = header.getAttribute("Type");
    
    if (!rep.equals(type)) {
      throw new ParsingException();
    } else if ((signatureAlg = header.getAttribute("SignatureDigestAlg")).length() == 0) {
      throw new ParsingException();
    } else if ((canonAlg = header.getAttribute("CanonAlg")).length() == 0) {
      throw new ParsingException();
    } else if ((rep = header.getAttribute("Version")).length() != 1) {
      throw new ParsingException();
    } else if (!Character.isDigit(rep.charAt(0))) {
      throw new ParsingException();
    }
    
    version = Integer.parseInt(rep);
    temp = header.getFirstChild();
    
    if (temp == null) {
      throw new ParsingException();
    } else if ((temp = temp.getFirstChild()) == null) {
      throw new ParsingException();
    } else if ((id = temp.getNodeValue()) == null) {
      throw new ParsingException();
    }
    
    temp = header.getFirstChild().getNextSibling();
    
    if (temp == null || temp.getNodeType() != Node.ELEMENT_NODE) {
      throw new ParsingException();
    }
    
    issuer = new AkentiIssuer((Element) temp);
    temp = temp.getNextSibling();
    
    if (temp == null || temp.getNodeType() != Node.ELEMENT_NODE) {
      throw new ParsingException();
    }
    
    Element validityPeriod = (Element) temp;
    
    notBefore = new UTCTime(validityPeriod.getAttribute("Begin"));
    notAfter = new UTCTime(validityPeriod.getAttribute("End"));
    doFromXML((Element) header.getNextSibling());
    child = signable.getNextSibling();
    
    if (child == null) {
      signature = "NO_SIGNATURE".getBytes();
    } else if ((temp = child.getFirstChild()) == null) {
      throw new ParsingException();
    } else if ((rep = temp.getNodeValue()) == null) {
      throw new ParsingException();
    } else {
      signature = Base64.decode(rep);
    }
  }
  
  /**
   */
  public final void
  toXML(Element cert) {
    Document doc = cert.getOwnerDocument();
    Element header = doc.createElement("Header");
    
    header.setAttribute("Type", getType());
    header.setAttribute("SignatureDigestAlg", getSignatureAlg());
    header.setAttribute("CanonAlg", getCanonAlg());
    header.setAttribute("Version", Integer.toString(version));
    
    Element temp = doc.createElement("UID");
    
    temp.appendChild(doc.createTextNode(id));
    header.appendChild(temp);
    temp = doc.createElement("Issuer");
    issuer.toXML(temp);
    header.appendChild(temp);
    temp = doc.createElement("ValidityPeriod");
    temp.setAttribute("Begin", notBefore.toString(UTCTime.XS_FORMAT));
    temp.setAttribute("End", notAfter.toString(UTCTime.XS_FORMAT));
    header.appendChild(temp);
    
    Element signable = doc.createElement("SignablePart");
    
    signable.appendChild(header);
    temp = doc.createElement(getType() + "Cert");
    doToXML(temp);
    signable.appendChild(temp);
    cert.appendChild(signable);
    temp = doc.createElement("Signature");
    temp.appendChild(doc.createTextNode(new String(Base64.encode(signature))));
    cert.appendChild(temp);
  }
  
  protected abstract void
  doWrite(AkentiOutputStream drain) throws IOException;
  
  protected abstract void
  doRead(AkentiInputStream source) throws IOException, ParseException;
  
  protected abstract
  void doToXML(Element element);
  
  protected abstract
  void doFromXML(Element elemnt) throws ParseException;
  
  // all set methods
  
  protected void setVersionNumber(int version) {
    this.version = version;
  }
  
  protected void setUniqueID(String id) {
    if (id.length() == 0) {
      throw new IllegalArgumentException();
    }
    
    this.id = id;
  }
  
  protected void setIssuer(AkentiIssuer issuer) {
      /**
    if (issuer == null) {
      throw new IllegalArgumentException();
    }
    */
    this.issuer = issuer;
  }
  
  protected void setNotBefore(UTCTime notBefore) {
    if (notBefore == null) {
      throw new IllegalArgumentException();
    }
    
    this.notBefore = notBefore;
  }
  
  protected void setNotAfter(UTCTime notAfter) {
    if (notAfter == null) {
      throw new IllegalArgumentException();
    }
    
    this.notAfter = notAfter;
  }
  
  public String paramString() {
    return "NEEDS TO BE IMPLEMENTED";
  }
  
  final public String toText() {
    StringBuffer sbuf = new StringBuffer();
    sbuf.append(getType() + " Certificate\n");
    sbuf.append("Version: V" + getVersion() + "\n");
    sbuf.append("Cert ID: " + getUniqueID() + "\n");
    sbuf.append("Issuer: \n");
    sbuf.append("  Name: " + getIssuer().getName() + "\n");
    sbuf.append("  CA: " + getIssuer().getGuarantor() + "\n");
    sbuf.append("Validity: \n");
    sbuf.append("  Not Before: " + getNotBefore().toFriendlyString() + "\n");
    sbuf.append("  Not After: " + getNotAfter().toFriendlyString() + "\n");
    doToText(sbuf);
    sbuf.append("Canon Algorithm: " + getCanonAlg() + "\n");
    sbuf.append("Signature Algorithm: " + getSignatureAlg() + "\n");
    String sig = new String(Base64.encode(getSignature()));
    sig = sig.replaceAll("\n", "\n           ");
    sbuf.append("Signature: " + sig + "\n");
    
    return sbuf.toString();
  }
  
  /**
   * Adds certificate specific information to StringBuffer.
   *
   **/
  abstract protected void doToText(StringBuffer sbuf);
  
}