package lbl.akenti.certs;

import java.io.IOException;
import java.text.ParseException;
import lbl.akenti.certs.elements.AkentiInputStream;
import lbl.akenti.certs.elements.AkentiOutputStream;
import lbl.akenti.certs.elements.AkentiPrincipal;
import lbl.akenti.certs.elements.AttributeInfo;
import lbl.akenti.certs.elements.Condition;
import lbl.akenti.certs.elements.Directory;
import lbl.akenti.certs.elements.DistinguishedName;
import lbl.akenti.util.ParsingException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * This class provides the implementation of an <code>attribute</code>
 * certificate.                                                      <BR>
 *
 * This certificate, once verified, will attest that the user
 * satisfies the attribute/value pair contained in this certificate. <BR>
 *
 * This certificate may contain a </code>Condition</code> that must
 * evaluate to <i>true</i> before the user is said to satisfy
 * the attribute/value pair.                                         <BR>
 *
 * These certificates are usually collected by the engine when
 * evaluating the boolean expression contained inside a
 * <code>UseCondition</code> certificate.                                  <BR>
 *
 * <p>
 *
 * The format for an attribute certificate is:
 *
 * <pre>
 * AttributeCertificate {
 *   type           // <code>Attribute</code>
 *   version        //  certificate's version  number
 *   id             //  certificate's unique id
 *   issuer         //  issuer of this certificate
 *   notBefore      //  start validity time
 *   notAfter       //  end validity time
 *   canonAlg       //
 *   signatureAlg   //  signature algorithm
 *   attribute      //  attribute's name
 *   value          //  attribute's value
 *   condition      //  constraint and the corresponding attributeInfos (optional)
 *   signature      //  digital signature
 * }
 * </pre>
 *
 * <p>
 *
 * Generating an attribute  certificate:
 *
 * <blockquote>
 *   AttributeCertGenerator gen = new AttributeCertGenerator();
 *
 *   gen.setIssuer(..);
 *   gen.setAttribute(..);
 *   gen.setValue(..);
 *   ...
 *   gen.sign(...).
 *
 *   AttributeCertificate cert = (AttributeCertificate) gen.generate();
 *
 * </blockquote>
 *
 * @see AttributeCertGenerator
 * @see AkentiCertificate
 *
 * @version 1.1 02/05/25
 * @author Srilekha Mudumbai
 * @author Abdelilah Essiari
 */
public final class AttributeCertificate extends AkentiCertificate {
  
  private AkentiPrincipal principal;
  private String attribute;
  private String value;
  private Condition condition;
  
  /**
   * Constructs this certificate using defalut values.
   */
  public AttributeCertificate() {
    super("Attribute");
    principal = new AkentiPrincipal();
    attribute = "NO_NAME";
    value = "NO_VALUE";
    condition = null;
  }
  
  /**
   */
  public AttributeCertificate(AkentiInputStream in) throws IOException, ParseException {
    super(in);
  }
  
  /**
   * Sets the name of the user.
   *
   * Note: This method is used during generation by the
   *       <code>AttributeCertGenerator</code>.
   */
  void setPrincipal(AkentiPrincipal principal) {
    if (principal == null) {
      throw new IllegalArgumentException();
    }
    
    this.principal = principal;
  }
  
  /**
   * Returns the name of the user.
   */
  public AkentiPrincipal getPrincipal() {
    return principal;
  }
  
  /**
   * Sets the <code>attribute</code>.
   *
   * Note: This method is used during generation by the
   *       <code>AttributeCertGenerator</code>.
   */
  void setAttribute(String attribute) {
    if (attribute.length() == 0) {
      throw new IllegalArgumentException();
    }
    
    this.attribute = attribute;
  }
  
  /**
   * Returns the <code>attribute</code>.
   */
  public String getAttribute() {
    return attribute;
  }
  
  /**
   * Sets the <code>value</code>.
   *
   * Note: This method is used during generation by the
   *       <code>AttributeCertGenerator</code>.
   */
  void setValue(String value) {
    if (value.length() == 0) {
      throw new IllegalArgumentException();
    }
    
    this.value = value;
  }
  
  /**
   * Returns the <code>value</code>.
   */
  public String getValue() {
    return value;
  }
  
  /**
   * Does this certificate contain some other criteria that
   * needs to be evaluated to true before we can claim that
   * the user satisfies the attribute/value pair.
   *
   * @see Condition
   */
  public boolean containsCondition() {
    return (condition != null);
  }
  
  /**
   * Sets the <code>condition</code>. This condition is optional
   * and it must be evaluated before we can claim that the
   * user satisfies this attribute/value pair.
   *
   * Note: This method is used during generation by the
   *       <code>AttributeCertGenerator</code>.
   *
   * @see Condition
   */
  void setCondition(Condition condition) {
    if (condition == null) {
      throw new IllegalArgumentException();
    }
    
    this.condition = condition;
  }
  
  /**
   * Returns the <code>condition</code>.
   *
   * @see Condition
   */
  public Condition getCondition() {
    return condition;
  }
  
  /**
   * Returns the <code>AttributeInfo</code>s contained in the
   * condition.
   *
   * @see Condition
   */
  public AttributeInfo[] getAttributeInfos() {
    return condition.getAttributeInfos();
  }
  
  /**
   * Returns the <code>constraint</code>s contained in the
   * condition.
   *
   * @see Condition
   */
  public String getConstraint() {
    return condition.getConstraint();
  }
  
  /**
   * Returns a hash that is used by the engine to
   * locate this certificate.
   */
  public String getHashString() {
    StringBuffer sbuf = new StringBuffer();
    
    sbuf.append(principal.getName().to500String());
    sbuf.append(principal.getGuarantor().to500String());
    sbuf.append(attribute);
    sbuf.append(value);
    return sbuf.toString();
  }
  /**
   * Returns an instance of this class.
   *
   * Note: Used by the <code>AkentiCertificateFactory<code>
   */
  protected Certificate newInstance() {
    return new AttributeCertificate();
  }
  
  /**
   * Writes this certificate to <code>AkentiOutputStream<code>.
   *
   * @see AkentiCertificate#writeObject(AkentiOutputStream)
   */
  protected void
  doWrite(AkentiOutputStream drain) throws IOException {
    principal.writeObject(drain);
    drain.write(' ');
    drain.write(attribute);
    drain.write(' ');
    drain.write(value);
    
    boolean haveCondition = (condition != null);
    
    drain.write(' ');
    drain.write(haveCondition);
    
    if (haveCondition) {
      drain.write(' ');
      condition.writeObject(drain);
    }
  }
  
  /**
   * Initializes this certificate from a <code>AkentiInputStream</code>.
   *
   * @see AkentiCertificate#readObject(AkentiInputStream)
   */
  protected void
  doRead(AkentiInputStream source) throws IOException, ParseException {
    condition = null;
    principal = new AkentiPrincipal(source);
    attribute = source.readString();
    value = source.readString();
    
    boolean haveCondition = source.readBoolean();
    
    if (haveCondition) {
      condition = new Condition(source);
    }
  }
  
  /**
   * Populates the XML element <code>attrCertInfo</code>.
   *
   * @see AkentiCertificate#toXML(Element)
   */
  protected void doToXML(Element attrCertInfo) {
    Document doc = attrCertInfo.getOwnerDocument();
    Element temp = doc.createElement("SubjectAndCA");
    
    principal.toXML(temp);
    attrCertInfo.appendChild(temp);
    temp = doc.createElement("AttrName");
    temp.appendChild(doc.createTextNode(attribute));
    attrCertInfo.appendChild(temp);
    temp = doc.createElement("AttrValue");
    temp.appendChild(doc.createTextNode(value));
    attrCertInfo.appendChild(temp);
    
    if (condition != null) {
      temp = doc.createElement("Condition");
      condition.toXML(temp);
      attrCertInfo.appendChild(temp);
    }
  }
  
  /**
   * Initializes this certificate from the XML element <code>attrCertInfo</code>.
   */
  protected void doFromXML(Element attrCertInfo) throws ParseException {
    Node child = attrCertInfo.getFirstChild();
    
    if (child == null || child.getNodeType() != Node.ELEMENT_NODE) {
      throw new ParsingException();
    }
    
    principal = new AkentiPrincipal((Element) child);
    child = child.getNextSibling();
    
    if (child == null || child.getNodeType() != Node.ELEMENT_NODE) {
      throw new ParsingException();
    }
    
    Node temp = child.getFirstChild();
    
    if (temp == null) {
      throw new ParsingException();
    } else if ((attribute = temp.getNodeValue()) == null) {
      throw new ParsingException();
    }
    
    child = child.getNextSibling();
    
    if ((temp = child.getFirstChild()) == null) {
      throw new ParsingException();
    } else if ((value = temp.getNodeValue()) == null) {
      throw new ParsingException();
    } else if ((child = child.getNextSibling()) == null) {
      return;
    } else if (child.getNodeType() != Node.ELEMENT_NODE) {
      throw new ParsingException();
    }
    
    condition = new Condition((Element) child);
  }
  
  protected void doToText(StringBuffer sbuf) {
    sbuf.append("  Subject: \n");
    sbuf.append("    Name: " + getPrincipal().getName() + "\n");
    sbuf.append("    CA: " + getPrincipal().getGuarantor() + "\n");
    sbuf.append("  AttributeValuePair: \n");
    sbuf.append("    " + getAttribute() + " = " + getValue() + "\n");
    
    if (containsCondition()) {
      sbuf.append("  Condition: \n");
      sbuf.append("    Constraint: " + getConstraint() + "\n");
      sbuf.append("    Atribute Infos: " + "\n");
      
      AttributeInfo[] ais = this.getAttributeInfos();
      for (int i=0; i < ais.length; i++) {
        
        sbuf.append("    " + ais[i].getAttribute() + " = " + ais[i].getValue());
        sbuf.append(", type = " + AttributeInfo.TYPES[ais[i].getType()] + "\n");
        
        if (ais[i].getType() == AttributeInfo.X509) {
          
          DistinguishedName[] cas = ais[i].getCAs();
          for (int j=0; j < cas.length; j++) {
            sbuf.append("      CA: " + cas[j] + "\n");
          }
          
        } else if (ais[i].getType() == AttributeInfo.AKENTI ||
        ais[i].getType() == AttributeInfo.EXT_AUTH) {
          
          AkentiPrincipal[] ap = null;
          if (ais[i].getType() == AttributeInfo.AKENTI ) {
            ap = ais[i].getIssuers();
          } else {
            ap = ais[i].getAuthorities();
          }
          
          for (int j=0; j < ap.length; j++) {
            sbuf.append("      Name: " + ap[j].getName() + "\n");
            sbuf.append("      CA: " + ap[j].getGuarantor() + "\n");
          }
          
          Directory[] dirs = ais[i].getDirectories();
          if (dirs.length != 0) {
            if (ais[i].getType() == AttributeInfo.AKENTI ) {
              sbuf.append("      Attribute Directories:\n");
            } else {
              sbuf.append("      Authority Directories:\n");
            }

            for (int k=0; k < dirs.length; k++) {
              sbuf.append("      " + dirs[k].toString() + "\n");
            }
          }
          
          if (ais[i].getType() == AttributeInfo.EXT_AUTH) {
            String[] args = ais[i].getArguments();
            
            if (args.length != 0) {
              sbuf.append("      Arguments:\n");
              
              for (int k=0; k < args.length; k++) {
                sbuf.append("        " + args[k] + "\n");
              }
            }
          }
        }
      }
    }
  }
  
}