package lbl.akenti.certs;

import java.io.IOException;
import java.security.GeneralSecurityException;
import java.security.KeyFactory;
import java.security.PublicKey;
import java.security.spec.X509EncodedKeySpec;
import java.text.ParseException;
import lbl.akenti.certs.elements.AkentiInputStream;
import lbl.akenti.certs.elements.AkentiOutputStream;
import lbl.akenti.certs.elements.AkentiPrincipal;
import lbl.akenti.certs.elements.AttributeInfo;
import lbl.akenti.certs.elements.ConditionalActions;
import lbl.akenti.certs.elements.DistinguishedName;
import lbl.akenti.util.Base64;
import lbl.akenti.util.ParsingException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
  * This class provides the implementation of a <code>capability</code>
  * certificate.
  *
  * <p>
  * The format for an attribute certificate is:
  * <pre>
  * AttributeCertificateImpl {
  *   type                  // <code>Capability</code>
  *   version               //  certificate's version  number
  *   id                    //  certificate's unique id
  *   issuer                //  issuer and signer of certificate
  *   notBefore             //  start validity time
  *   notAfter              //  end validity time
  *   signatureAlg          //  signature algorithm
  *   principal             //  principal
  *   publicKey             //  principal's public key (version 2)
  *   resource              //  name of resource
  *   unConditionalActions  //  actions with no conditions attached to them
  *   conditionalActions    //  actions that would be granted once the
  *                         //    condition attached to them evalutes to true
  *   signature             //  digital signature
  * }
  * </pre>
  *
  * <p>
  * Generating a capability certificate:
  *
  * <blockquote>
  *   CapabilityCertificateGenerator gen;
  *
  *   gen.setIssuer(..);
  *   ...
  *   gen.sign(...).
  *
  *   AkentiCertificate cert = gen.generate();
  * </blockquote>
  *
  * @see CapabilityCertGenerator
  * @see ConditionalActions
  * @see AkentiPrincipal
  *
  * @version 1.1 02/05/25
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  */
public final class CapabilityCertificate extends AkentiCertificate {

  private AkentiPrincipal principal;

  private PublicKey publicKey;

  private String resource;

  private String[] unConditionalActions;

  private ConditionalActions[] conditionalActions;

  public CapabilityCertificate() {
    super("Capability");
    principal = new AkentiPrincipal();
    resource = "NO_RESOURCE";
    unConditionalActions = new String[] { "NO_ACTION" };
    conditionalActions = new ConditionalActions[0];
  }

  public CapabilityCertificate(AkentiInputStream in) throws IOException, ParseException {
    super(in);
  }

  public String getHashString() {
    StringBuffer sbuf = new StringBuffer(resource);

    sbuf.append(principal.getName().to500String());
    sbuf.append(principal.getGuarantor().to500String());
    return sbuf.toString();
  }

  void setResource(String resource) {
    this.resource = resource;
  }

  public String getResource() {
    return resource;
  }

  void setPrincipal(AkentiPrincipal principal) {
    this.principal = principal;
  }

  public AkentiPrincipal getPrincipal() {
    return principal;
  }

  void setPublicKey(PublicKey publicKey) {
    this.publicKey = publicKey;
  }
 
  public PublicKey getPublicKey() {
    return publicKey;
  }

  void setUnConditionalActions(String[] unConditionalActions) {
    this.unConditionalActions = unConditionalActions;
  }

  public String[] getUnConditionalActions() {
    return unConditionalActions;
  }

  void setConditionalActions(ConditionalActions[] condiditionalActions) {
    this.conditionalActions = condiditionalActions;
  }

  public ConditionalActions[] getConditionalActions() {
    return conditionalActions;
  }

  protected Certificate newInstance() {
    return new CapabilityCertificate();
  }

  protected void doWrite(AkentiOutputStream drain) throws IOException {
    principal.writeObject(drain);
    
    if (version == 2) {
      byte []der = publicKey.getEncoded();
 
      drain.write(' ');
      drain.write(der.length);
      drain.write(' ');
      drain.write(der);
    }

    drain.write(' ');
    drain.write(resource);

    drain.write(' ');
    drain.write(unConditionalActions.length);

    for (int i = 0; i < unConditionalActions.length; i++) {
      drain.write(' ');
      drain.write(unConditionalActions[i]);
    }

    drain.write(' ');
    drain.write(conditionalActions.length);

    for (int i = 0; i < conditionalActions.length; i++) {
      conditionalActions[i].writeObject(drain);
    }
  }

  protected void 
  doRead(AkentiInputStream source) throws IOException, ParseException {
    principal = new AkentiPrincipal(source);
    
    if (version == 2) {
      try {
        // TODO: HOW ABOUT DSA PUBLICK KEY?
 
        byte []der = source.readBytes(source.readInt());
        X509EncodedKeySpec keySpec = new X509EncodedKeySpec(der);
        KeyFactory keyFactory = KeyFactory.getInstance("RSA");
        publicKey = keyFactory.generatePublic(keySpec);
      } catch(GeneralSecurityException ex) {
        throw new ParsingException(ex);
      }
    }

    resource = source.readString();

    int len = source.readInt();
    
    if (len < 0) {
      throw new ParsingException();
    }

    unConditionalActions = new String[len];

    for (int i = 0; i < len; i++) {
      unConditionalActions[i] = source.readString();
    }

    if ((len = source.readInt()) < 0) {
      throw new ParsingException();
    }

    conditionalActions = new ConditionalActions[len];

    for (int i=0; i < len; i++) {
      conditionalActions[i] = new ConditionalActions(source);
    }
     
    if (unConditionalActions.length == 0 && conditionalActions.length == 0) {
      throw new ParsingException();
    }
  }

  protected void doToXML(Element capCertInfo) {
    Document doc = capCertInfo.getOwnerDocument();
    Element temp = doc.createElement("ResourceName");

    temp.appendChild(doc.createTextNode(resource));
    capCertInfo.appendChild(temp);
    temp = doc.createElement("SubjectAndCA");
    principal.toXML(temp);
    capCertInfo.appendChild(temp);

    byte []der = publicKey.getEncoded();

    temp = doc.createElement("PublicKey");
    temp.appendChild(doc.createTextNode(new String(Base64.encode(der))));
    capCertInfo.appendChild(temp);

    for (int i = 0; i < unConditionalActions.length; ++i) {
      temp = doc.createElement("Action");
      temp.appendChild(doc.createTextNode(unConditionalActions[i]));
      capCertInfo.appendChild(temp);
    } 

    for (int i = 0; i < conditionalActions.length; i++) {
      temp = doc.createElement("ConditionalActions");
      conditionalActions[i].toXML(temp);
      capCertInfo.appendChild(temp);
    } 
  }

  protected void doFromXML(Element capCertInfo) throws ParseException {
    Node child = capCertInfo.getFirstChild();

    if (child == null || child.getNodeType() != Node.ELEMENT_NODE) {
      throw new ParsingException();
    }

    Node temp = child.getFirstChild();

    if (temp == null) {
      throw new ParsingException();
    } else if ((resource = temp.getNodeValue()) == null) {
      throw new ParsingException();
    } else if ((child = child.getNextSibling()) == null) {
      throw new ParsingException();
    } else if (child.getNodeType() != Node.ELEMENT_NODE) {
      throw new ParsingException();
    }

    principal = new AkentiPrincipal((Element) child);

    if ((child = child.getNextSibling()) == null) {
      throw new ParsingException();
    } else if (child.getNodeType() != Node.ELEMENT_NODE) {
      throw new ParsingException();
    } else if ((temp = child.getFirstChild()) == null) {
      throw new ParsingException();
    }

    String base64 = temp.getNodeValue();

    if (base64 == null) {
      throw new ParsingException();
    }

    byte []der = Base64.decode(base64);

    try {
      X509EncodedKeySpec keySpec = new X509EncodedKeySpec(der);
      KeyFactory keyFactory = KeyFactory.getInstance("RSA");
      publicKey = keyFactory.generatePublic(keySpec);
    } catch(GeneralSecurityException ex) {
      throw new ParsingException(ex);
    }

    NodeList list = capCertInfo.getElementsByTagName("Action");
    int len = list.getLength();

    unConditionalActions = new String[len];

    for (int i = 0; i < len; ++i) {
      child = list.item(i);

      if ((unConditionalActions[i] = child.getNodeValue()) == null) {
        throw new ParsingException();
      }
    }

    list = capCertInfo.getElementsByTagName("ConditionalActions");
    len = list.getLength();
    conditionalActions = new ConditionalActions[len];

    for (int  i = 0; i < len; i++) {
      child = list.item(i);
      
      if (child.getNodeType() != Node.ELEMENT_NODE) {
        throw new ParsingException();
      }

      conditionalActions[i] = new ConditionalActions((Element) child);
    }

    if (unConditionalActions.length == 0 && conditionalActions.length == 0) {
      throw new ParsingException();
    }
  }

  public void doToText(StringBuffer sbuf) {
    sbuf.append("  Subject: \n");
    sbuf.append("    Name: " + principal.getName().toString() + "\n");
    sbuf.append("    CA: " + principal.getGuarantor().toString() + "\n");
    sbuf.append("  Resource: " + resource + "\n");
    
    String[] uactions = getUnConditionalActions();
    
    if ( uactions.length != 0) {
      sbuf.append("  Granted Actions: \n");
      sbuf.append("    " + uactions[0]);
      
      for (int i=1; i < uactions.length; i++) {
        sbuf.append(", " + uactions[i]);
      }
      sbuf.append("\n");
    }
    
    ConditionalActions[] cas = getConditionalActions();
    
    if (cas.length != 0) {
      sbuf.append("  Conditional Actions: \n");
      
      for (int i=0; i < cas.length; i++) {
        int number = i+1;
        sbuf.append("  " + number + ".\n");
        String critical = (cas[i].isCritical()) ? "true" : "false";
        sbuf.append("    Critical: " + critical + "\n");
        sbuf.append("    Constraint: " + cas[i].getConstraint() + "\n");
        
        String[] ga = cas[i].getActions();
        if (ga.length != 0) {
          
          sbuf.append("      Actions: \n");
          sbuf.append("        " + cas[0].getConstraint());
          
          for (int j=1; j < ga.length; j++) {
            sbuf.append(", " + cas[i].getConstraint());
          }
          sbuf.append("\n");
        }
        
        sbuf.append("      Attribute Info(s): \n");
        AttributeInfo[] ais = cas[i].getAttributeInfos();
        for (int j=0; j < cas.length; j++) {
          sbuf.append("        attribute = " + ais[j].getAttribute());
          sbuf.append(" , value = " + ais[j].getValue());
          sbuf.append(" , type = " + AttributeInfo.TYPES[ais[j].getType()]);
          sbuf.append("\n");
          
          if (ais[j].getType() == AttributeInfo.X509) {
            DistinguishedName[] acas = ais[j].getCAs();
            
            for (int k=0; k < acas.length; k++) {
              sbuf.append("          CA: " + acas[k] + "\n");
            }
          } else if (ais[j].getType() == AttributeInfo.AKENTI || ais[j].getType() == AttributeInfo.EXT_AUTH) {
            AkentiPrincipal[] aps = null;
            if (ais[j].getType() == AttributeInfo.AKENTI) {
              aps = ais[j].getIssuers();
            } else {
              aps = ais[j].getAuthorities();
            }
            
            for (int k=0; k < aps.length; k++) {
              sbuf.append("          Name: " + aps[k].getName() + "\n");
              sbuf.append("          CA: " + aps[k].getGuarantor() + "\n");
            }
          }
        }
      }
    }
  }
}
