package lbl.akenti.certs.elements;

import java.io.IOException;
import java.io.InputStream;
import java.io.StreamTokenizer;

/**
  * An <code>AkentiInputStream</code> is used to deserialize objects
  * from a byte stream that was written using the conventions defined
  * in <code>AkentiOutputStream</code>.<BR>           
  * 
  * This class provides input operations for primitive types, strings, and 
  * byte arrays.
  *
  * @see AkentiOutputStream
  *
  * @version 1.1 02/05/01
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  **/
public final class AkentiInputStream {

 private final static
    char[][] wordChars = 
      { {'a', 'z'},
        {'A', 'Z'},
        {128+32, 255},
        {'/', '/'},
        {'0', '9'},
        {'_', '_'},
        {'[', '`'},
        {'{', '~'},
        {':', '@'},
        {'!', '.'}
      };

    char[][] ordinaryChars=
      { {'.','.'},
        {'0','9'},
        {'-','-'}
      };

  private StreamTokenizer st;

  private InputStream is;

  /**
   * Constructs this AkentiInputStream.
   * @param is the input
   */
  public AkentiInputStream(InputStream is) {
    // DO NOT USE InputStreamReader.
    st = new StreamTokenizer(this.is = is);
  }

  /**
   * Reads a sequence of characters.
   * @throws IOException if the eof is reached
   * @return the next string
   */
  public String readString() throws IOException {
    st.resetSyntax();

    for (int i=0; i < ordinaryChars.length; i++) {
      st.ordinaryChars(ordinaryChars[i][0], ordinaryChars[i][1]);
    }

    for (int i=0; i < wordChars.length; i++) {
      st.wordChars(wordChars[i][0], wordChars[i][1]);
    }

    st.whitespaceChars(0, ' ');
    st.eolIsSignificant(false);

    StringBuffer sb = new StringBuffer();
    int type = st.nextToken();

    if (type == StreamTokenizer.TT_EOF) {
      throw new IOException("reached eof");
    }

    while (type != StreamTokenizer.TT_EOF && 
           st.sval.charAt(st.sval.length() - 1) == '\\') { 

      String temp = st.sval;

      type = st.nextToken();

      if (type == StreamTokenizer.TT_EOF) {
        sb.append(temp);
      } else {
        sb.append(temp.substring(0, temp.length() - 1));
        sb.append(' ');
      }
    } 

    if (type != StreamTokenizer.TT_EOF) {
      sb.append(st.sval);
    }

    return sb.toString();
  }

  /**
   * Reads a short integer.
   * @throws IOException if the eof is reached of if the next output is
   * not a number.
   * @return the next short
   */
  public short readShort() throws IOException {
    return (short) readDouble();
  }

  /**
   * Reads an integer.
   * @throws IOException if the eof is reached of if the next output is
   * not a number.
   * @return the next int
   */
  public int readInt() throws IOException {
    return (int) readDouble();
  }

  /**
   * Reads a long integer.
   * @throws IOException if the eof is reached of if the next output is
   * not a number.
   * @return the next long
   */
  public long readLong() throws IOException {
    return (long) readDouble();
  }

  /**
   * Reads a boolean.
   * @throws IOException if the eof is reached of if the next output is
   * not a number.
   * @return the next boolean
   */
  public boolean readBoolean() throws IOException {
    double d = readDouble();

    if (d == 1) {
      return true;
    } else if (d == 0) {
      return false;
    } else {
      throw new IOException("unable to read boolean");
    }
  }

  /**
   * Reads a byte.
   * @throws IOException if the eof is reached of if the next output is
   * not a number.
   * @return the next byte
   */
  public byte readByte() throws IOException {
    return readBytes(1)[0];
  }

  /**
   * Reads <CODE>len</CODE> bytes.
   * @param len the number of bytes to read
   * @throws IOException if the eof is reached
   * @return the next <code>len</code> bytes
   */
  public byte[] readBytes(int len) throws IOException {
    byte[] buf = new byte[len];

    if (is.read(buf, 0, len) != len) {
      throw new IOException("unable to read " + len + " bytes");
    }
    
    return buf;
  }

  /**
   * Reads one character.
   * @throws IOException if the eof is reached of if the next output is
   * not a number.
   * @return the next character
   */
  public char readChar() throws IOException {
    return (char) readByte();
  }

  /**
   * Reads a float.
   * @throws IOException if the eof is reached of if the next output is
   * not a number.
   * @return the next float
   */
  public float readFloat() throws IOException {
    return (float) readDouble();
  }

  /**
   * Reads a double.
   * @throws IOException if the eof is reached of if the next output is
   * not a number.
   * @return the next double
   */
  public double readDouble() throws IOException {
    st.parseNumbers();

    if (st.nextToken() == StreamTokenizer.TT_EOF) {
      throw new IOException("end of file reached");
    }

    if (st.ttype != StreamTokenizer.TT_NUMBER) {
      throw new IOException("unable to read number");
    } 
    
    return st.nval;
  }
}
