package lbl.akenti.certs.elements;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.security.GeneralSecurityException;
import java.security.cert.CertificateEncodingException;
import java.security.cert.X509Certificate;
import java.text.ParseException;
import lbl.akenti.util.Base64;
import lbl.akenti.util.ParsingException;
import lbl.akenti.util.X509CertReader;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
  * A <code>CertificateAuthorityInfo</code> holds a ca's X509 chain.
  * In addition, although not required, directories where to 
  * search for identity certs and crls.
  *
  * CertificateAuthorityInfo {                      <br>
  *   vector&lt;Certificate&gt;        certs;       <br>
  *   vector&lt;Directory&gt;         idDirs;       <br>
  *   vector&lt;Directory&gt;        crlDirs;       <br>
  * }                                               <br>
  *                                                 <br>
  *
  * The format of writeObject is following :                <br>
  *   certCount cert+ idDirCount idDir* crlDirCount crlDir* <br>
  *
  * Where cert is the DER encoding of a x509 certificate,
  *       + means one or more
  *       * means 0 or more
  *
  * @see AkentiObject
  * @see Directory
  *
  * @version 1.1 02/05/01
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  **/
public final class CertificateAuthorityInfo extends AkentiObject {

  // CA's X509 chain
  private X509Certificate[] certs;

  // list of locators where X509 certificates may be found */
  private Directory[] identityDirs = new Directory[0];

  // list of locators where X509 revocation lists may be found.
  private Directory[] crlDirs = new Directory[0];

  /**
    * Constructs this <code>CertificateAuthorityInfo</code>.
    *
    * Note: The ca's certificate is at the end of the array.
    *
    * @param certs CA's X509 chain of certificates.
    */
  public CertificateAuthorityInfo(X509Certificate[] certs) {
    this(certs, null);
  }

  /**
    * Constructs this <code>CertificateAuthorityInfo</code>.
    *
    * Note: The ca's certificate is at the end of the array.
    *
    * @param certs ca's x509 chain.
    * @param idDirs  locators where X509 certs can be found.
    */
  public CertificateAuthorityInfo(X509Certificate[] certs, Directory[] idDirs) {
    this(certs, idDirs, null);
  }

  /**
    * Constructs this <code>CertificateAuthorityInfo</code>.
    *
    * @param certs ca's x509 chain.
    * @param idDirs locators where X509 certs can be found.
    * @param crlDirs locators where CRLs can be found.
    */
  public 
  CertificateAuthorityInfo(X509Certificate[] certs, 
                           Directory[] idDirs,
                           Directory[] crlDirs) {
    if (certs.length == 0) {
      throw new IllegalArgumentException();
    }
    
    this.certs = certs;
    this.identityDirs = idDirs == null ? new Directory[0] : idDirs;
    this.crlDirs = crlDirs == null ? new Directory[0] : crlDirs;
  }

  /**
    * Constructs this <code>CertificateAuthorityInfo</code>.
    *
    * @param source stream used to initialize this object.
    */
  public
  CertificateAuthorityInfo(AkentiInputStream source) 
    throws IOException, ParseException {
    readObject(source);
  }

  /**
    * Constructs this <code>CertificateAuthorityInfo</code>.
    *
    * @param element XML element used to parse this object.
    */
  public
  CertificateAuthorityInfo(Element element) throws ParseException {
    fromXML(element);
  }

  /**
    * Returns the CA's chain.
    */
  public X509Certificate[] getChain() {
    return certs;
  }

  /**
    * Returns the CA's distinguished name in X500 format.
    */
  public String getName() {
    return getDistinguishedName().to500String();
  }

  /**
    * Returns the CA's <code>DistinguishedName</code>.
    */
  public DistinguishedName getDistinguishedName() {
    try {
      return new DistinguishedName(certs[certs.length - 1].getSubjectDN());
    } catch(ParsingException ex) {
      throw new RuntimeException();
    }
  }

  /**
    * Returns the identity locators.
    */
  public Directory[] getIdentityDirectories() {
    return identityDirs;
  }

  /**
    * Returns the CRL locators.
    */
  public Directory[] getCRLDirectories() {
    return crlDirs;
  }

  /**
    * Returns a string representation of this <code>Directory</code> that can be
    * used by gui widgets.
    */
  public String toString() {
    return getName();
  }

  /**
    * Writes this <code>CertificateAuthorityInfo</code> to a <code>AkentiOutputStream</code>.
    */
  public void writeObject(AkentiOutputStream drain) throws IOException {
    drain.write(certs.length); 

    for (int i=0; i < certs.length; i++) {
      try {
        byte[] der = certs[i].getEncoded();

        drain.write(' ');
        drain.write(der.length);
        drain.write(' ');
        drain.write(der);
      } catch(CertificateEncodingException ex) {
        throw new RuntimeException();
      }
    }

    drain.write(' ');
    drain.write(identityDirs.length);

    for (int i=0; i < identityDirs.length; i++) {
      drain.write(' ');
      identityDirs[i].writeObject(drain);
    }

    drain.write(' ');
    drain.write(crlDirs.length);

    for (int i=0; i < crlDirs.length; i++) {
      drain.write(' ');
      crlDirs[i].writeObject(drain);
    }
  }

  /**
    * Initializes this <code>CertificateAuthorityInfo</code> from a <code>AkentiInputStream</code>.
    */
  private void readObject(AkentiInputStream source) 
    throws IOException, ParseException {
    int size = source.readInt();
    
    if (size <= 0) {
      throw new ParsingException("cacerts.size <= 0");
    }

    X509CertReader certReader = new X509CertReader();

    certs = new X509Certificate[size];
     
    for (int i=0; i < size; i++) {
      try {
        byte[] der = source.readBytes(source.readInt());

        certs[i] = certReader.readX509Cert(new ByteArrayInputStream(der));
      } catch (GeneralSecurityException ex) {
        throw new ParsingException(ex);
      }
    }

    if ((size = source.readInt()) < 0) {
      throw new ParsingException("idDirs.size < 0");
    }

    identityDirs = new Directory[size];

    for (int i=0; i < size; i++) {
      identityDirs[i] = new Directory(source);
    }
    
    if ((size = source.readInt()) < 0) {
      throw new ParsingException("crls.size < 0");
    }

    crlDirs = new Directory[size];

    for (int i=0; i < size; i++) {
      crlDirs[i] = new Directory(source);
    }
  }

  /**
    * Writes this <code>CertificateAuthorityInfo</code> to a <code>AkentiOutputStream</code>.
    */
  public void toXML(Element caInfo) {
    Document doc = caInfo.getOwnerDocument();
    Element temp = doc.createElement("CADN");
  
    getDistinguishedName().toXML(temp);
    caInfo.appendChild(temp);

    for (int i=0; i < certs.length; i++) {
      try {
        String enc = new String(Base64.encode(certs[i].getEncoded()));
        int tempsize = enc.length() -1;

        if (enc.charAt(tempsize-1) == '\n') {
          enc = enc.substring(0, tempsize-1);
        }

        temp = doc.createElement("X509Certificate");
        temp.appendChild(doc.createTextNode(enc));
        caInfo.appendChild(temp);
      } catch (GeneralSecurityException ex) {
        throw new RuntimeException();
      }
    }

    if (identityDirs.length > 0) {
      Element idDirs = doc.createElement("IdDirs");

      for (int i=0; i < identityDirs.length; i++) {
        temp = doc.createElement("URL");
        identityDirs[i].toXML(temp);
        idDirs.appendChild(temp);
      }

      caInfo.appendChild(idDirs);
    }

    if (crlDirs.length > 0) {
      Element revokeDirs = doc.createElement("CRLDirs");

      for (int i=0; i < crlDirs.length; i++) {
        temp = doc.createElement("URL");
        crlDirs[i].toXML(temp);
        revokeDirs.appendChild(temp);
      }

      caInfo.appendChild(revokeDirs);
    }
  }

  /**
    * Initializes this <code>Directory</code> from a <code>AkentiInputStream</code>.
    */
  private void fromXML(Element caInfo) throws ParseException {
    Node child = caInfo.getFirstChild();
    
    if (child == null || child.getNodeType() != Node.ELEMENT_NODE) {
      throw new ParsingException();
    }
  
    // XXX
    DistinguishedName dn = new DistinguishedName((Element) child);
    NodeList list = caInfo.getElementsByTagName("X509Certificate");
    int len = list.getLength();

    if (len == 0) {
      throw new ParsingException();
    }

    certs = new X509Certificate[len];

    Node temp;

    for (int i = 0; i < len; i++) {
      child = list.item(i);
      
      if ((temp = child.getFirstChild()) == null) {
        throw new ParsingException();
      }

      String certstr = temp.getNodeValue();

      if (certstr == null) {
        throw new ParsingException();
      }

      certstr = certstr.trim();

      int startStr = 0; int endStr = certstr.length();

      /* 
       * XXX
       * Check to see if there are PEM tags around the cert 
       * and if so, skip them
       */
      if (certstr.startsWith("-----BEGIN CERTIFICATE-----")) {
        startStr = 27;
      }

      if (certstr.endsWith("-----END CERTIFICATE-----")) {
        endStr = endStr - 25;
      }

      byte[] certbytes = Base64.decode((certstr.substring(startStr,endStr)).toCharArray());

      try {
        certs[i] = new X509CertReader().readX509Cert(new ByteArrayInputStream(certbytes));
      } catch (IOException ex) {
        throw new RuntimeException();
      } catch (GeneralSecurityException ex) {
        throw new ParsingException(ex);
      }
    }

    child = child.getNextSibling();
    
    if (child == null) {
      identityDirs = crlDirs = new Directory[0];
      return;
    } else if (child.getNodeType() != Node.ELEMENT_NODE) {
      throw new ParsingException();
    }

    if (child.getNodeName().equals("IdDirs")) {
      list = ((Element) child).getElementsByTagName("URL");

      if ((len = list.getLength()) == 0) {
        throw new ParsingException();
      }

      identityDirs = new Directory[len]; 

      for (int i = 0; i < len; i++) {
        identityDirs[i] = new Directory((Element) list.item(i));
      }
      
      if ((child = child.getNextSibling()) == null) {
        return;
      } else if (child.getNodeType() != Node.ELEMENT_NODE) {
        throw new ParsingException();
      }
    }

    list = ((Element) child).getElementsByTagName("URL");

    if ((len = list.getLength()) == 0) {
      throw new ParsingException();
    }

    crlDirs = new Directory[len]; 

    for (int i = 0; i < len; i++) {
      crlDirs[i] = new Directory((Element) list.item(i));
    }
  }

  /**
    * Returns debugging info.
    */
  public String paramString() {
    StringBuffer sbuf = new StringBuffer();

    sbuf.append("[");

    try { 
      sbuf.append("X509Certificate");
      sbuf.append("(");
      sbuf.append(1);
      sbuf.append(")");
      sbuf.append("=");
      sbuf.append(Base64.encode(certs[0].getEncoded()));
    } catch (CertificateEncodingException ex) {
      throw new RuntimeException();
    }

    for (int i=1; i < certs.length; i++) {
      try {
	sbuf.append(",");
        sbuf.append("X509Certificate");
	sbuf.append("(");
	sbuf.append(i+1);
        sbuf.append(")");
        sbuf.append("=");
        sbuf.append(Base64.encode(certs[i].getEncoded()));
      } catch (CertificateEncodingException ex) {
        throw new RuntimeException();
      }
    }

    sbuf.append("ID DIR");
    sbuf.append("(");
    sbuf.append(1);
    sbuf.append(")");
    sbuf.append("=");
    sbuf.append(identityDirs[0].paramString());

    for (int i=1; i < identityDirs.length; i++) {
      sbuf.append(",");
      sbuf.append("ID DIR");
      sbuf.append("(");
      sbuf.append(i+1);
      sbuf.append(")");
      sbuf.append("=");
      sbuf.append(identityDirs[i].paramString());
    }

    sbuf.append("CRL DIR");
    sbuf.append("(");
    sbuf.append(1);
    sbuf.append(")");
    sbuf.append("=");
    sbuf.append(crlDirs[0].paramString());

    for (int i=1;i < crlDirs.length; i++) {
      sbuf.append(",");
      sbuf.append("CRL DIR");
      sbuf.append("(");
      sbuf.append(i+1);
      sbuf.append(")");
      sbuf.append("=");
      sbuf.append(crlDirs[i].paramString());
    }

    sbuf.append("]");
    return sbuf.toString();
  }
}
