package lbl.akenti.certs.elements;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.ParseException;
import lbl.akenti.util.ParsingException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
  * A <code>Directory</code> contains a string representation of a URL.
  * It was introduced in addition to the URL class for two reasons.
  *
  * 1) Policy and UseConditition Certificates contain urls pointing to
  *    directories in which to search for other certificates. Since
  *    these urls are contained in the signable part of a certificate,
  *    we must maintain the exact string representation of the
  *    url.
  *
  *    Example: "http://hostname:8080/filename
  *             "http://hostname:80/filename/
  *             "http://hostname/filename/
  *    The URL class can parse this but the URL.toString() will
  *    not return the exact string.  Should it?
  *
  * 2) A <code>URLHandler</code> may not be installed. And not being able
  *    to parse a particular url should not trigger an exception
  *    when reading a certificate.
  *
  * 3) A <code>Directory</code> object provides us a cleaner and a more flexible
  *    solution for the future.
  *
  * @see AkentiObject
  *
  * @version 1.1 02/05/01
  *
  * @author Srilekha Mudumbai
  * @author Abdelilah 
  */
public final class Directory extends AkentiObject {

  private String url;

  /**
    * Constructs this <code>Directory</code> using a default url.
    */
  public Directory() {
    this("NO_PROT://NOHOST:NOPORT");
  }

  /**
    * Constructs this <code>Directory</code>.
    */
  public Directory(String url) {
    if (url.length() == 0) {
      throw new IllegalArgumentException();
    }

    this.url = url;
  }

  /**
    * Constructs this <code>Directory</code>.
    *
    * @param source stream used to initialize this directory.
    */
  public Directory(AkentiInputStream source) throws IOException, ParseException {
    readObject(source);
  }

  /**
    * Constructs this <code>Directory</code>.
    *
    * @param element XML element used to parse this object.
    */
  public Directory(Element element) throws ParseException {
    fromXML(element);
  }

  /**
    * Tests for equality. Useful in containers such as vectors or sets.
    */
  public boolean equals(Object obj) {
    if (obj instanceof Directory) {
      Directory dir = (Directory) obj;

      return dir.url.equals(url);
    }

    return false;
  }

  /**
    * Computes a hash that can be used by hash containers such as
    * a Hashtable.
    */
  public int hashCode() {
    return url.hashCode();
  }

  /**
    * Parses this Directory.
    *
    * @return a URL object.
    */
  public URL parseDirectory() throws MalformedURLException {
    return new URL(url);
  }

  /**
    * Writes this <code>Directory</code> to a <code>AkentiOutputStream</code>.
    */
  public void 
  writeObject(AkentiOutputStream drain) throws IOException {
    drain.write(url);
  }

  /**
    * Initializes this <code>Directory</code> from a <code>AkentiInputStream</code>.
    */
  private void 
  readObject(AkentiInputStream source) throws IOException, ParseException {
    url = source.readString();
  }

  /**
    * Initializes this <code>Directory</code> from the XML element <code>dir</code>.
    */
  public void toXML(Element dir) {
    Document doc = dir.getOwnerDocument();

    dir.appendChild(doc.createTextNode(url));
  }

  /**
    * Populates the XML element <code>dir</code>.
    */
  private void fromXML(Element dir) throws ParseException {
    Node child = dir.getFirstChild();

    if (child == null) {
      throw new ParsingException();
    } else if ((url = child.getNodeValue()) == null) {
      throw new ParsingException();
    }
  }

  /**
    * Returns a string representation of this <code>Directory</code> that can be
    * used by gui widgets.
    */
  public String toString() {
    return url;
  }

  /**
    * Returns debugging info.
    */
  public String paramString() {
    StringBuffer sbuf = new StringBuffer();

    sbuf.append("[");
    sbuf.append("url=");
    sbuf.append(url);
    sbuf.append("]");
    return sbuf.toString();
  }
}
