package lbl.akenti.gen.controller;

import java.util.BitSet;

import lbl.akenti.util.Util;
import lbl.akenti.certs.AkentiCertificateFactory;
import lbl.akenti.certs.AkentiCertGenerator;
import lbl.akenti.certs.AkentiCertificate;
import lbl.akenti.certs.elements.AkentiPrincipal;
import lbl.akenti.certs.elements.AkentiIssuer;

import lbl.akenti.util.P12Reader;

import lbl.akenti.gen.common.GenInfo;
import lbl.akenti.gen.common.GenCa;
import lbl.akenti.gen.common.CredentialInfo;
import lbl.akenti.gen.controller.LBLResourceFrame;

import java.security.KeyStore;
import java.security.SecureRandom; 
import lbl.akenti.util.X509CertReader;
import java.security.cert.X509Certificate;
import java.net.URL;
import java.util.Properties;

import java.net.URL;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.BufferedInputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.LineNumberReader;
import java.io.IOException;
import java.io.OutputStream;
import java.net.Socket;
import java.io.BufferedOutputStream;

import com.sun.net.ssl.KeyManagerFactory;
import com.sun.net.ssl.TrustManagerFactory;
import com.sun.net.ssl.SSLContext;
import com.sun.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.SSLSession;
import javax.net.SocketFactory;

import java.security.NoSuchAlgorithmException;
import java.security.GeneralSecurityException;
import java.net.MalformedURLException;
import java.net.URLConnection;
import java.io.IOException;
import java.security.KeyStoreException;
import java.security.cert.CertificateException;
import java.security.KeyManagementException;

import javax.swing.JDialog;


/**
  * Provides common functionality to all controllers.
  */
abstract public class AbstractController extends Controller {
  protected GenInfo info;

  protected LBLFrame []frames; // array of frames in the GUI

  protected int whichFrame;    // index of frame that is currently displayed

  protected BitSet bits;       // one bit for each frame, if a bit is 0 
                               // loadInputsIntoFrameAt is called
                               // The first bit is unused since we always show
                               // the first frame.

  protected AkentiCertGenerator gen; // This must be set by the subclass

  boolean init = false;              // ssl initialization
  
  /**
    * constructor 
    *
    * @param  numOfFrames number of frames in the whole interface
    */
  protected AbstractController(int numOfFrames) {
    frames = new LBLFrame[numOfFrames];
    bits = new BitSet(numOfFrames);
    whichFrame = 0;

    //Allow ldap urls
    java.net.URL.setURLStreamHandlerFactory(new java.net.URLStreamHandlerFactory() {
      public java.net.URLStreamHandler
        createURLStreamHandler(String protocol) {

        if (protocol.equalsIgnoreCase("ldap")) {
          return new java.net.URLStreamHandler() {
            protected java.net.URLConnection
            openConnection(java.net.URL url) throws java.io.IOException {
              throw new java.io.IOException("open ldap connection unsupported");
            }
          };
        }

        return null;
      }
    });

    //Allow https urls
    Properties props = System.getProperties();

    props.put("java.protocol.handler.pkgs", "com.sun.net.ssl.internal.www.protocol");
  }

  /**
    * Subclasses must return an <i>LBLFrame</i> that will
    * corresspond to the index <i>ind</i>.
    *
    * Note This method will be called for each index.
    *
    * @param index the index of the requested frame
    */
  abstract protected LBLFrame doGetFrameAt(int ind);

  /**
    * Subclasses must load inputs to frame at index <i>nextFrameIndex</i>.
    * 
    * Note this method is only called if the bit * corresponding to this 
    * frame is not set. 
    *
    * @return true if this frame does not wish to be loaded.
    *         false otherwise.
    *
    * @see showNext()
    * @see clearBits()
    */
  abstract protected boolean 
  loadInputsIntoFrameAt(int nextFrameIndex) throws Exception;
    
  /**
    * Subclasses must return the help message corresponding
    * to the frame at index <i>ind</i>.
    *
    */
  abstract protected String getHelpMessage(int currentFrameIndex);  

  /**
    * Sets up the very first frame to be displayed.
    */
  protected  void start() {
    getFrameAt(0).show();
  }

  /**
    */
  final protected LBLFrame getFrameAt(int index) {
    if (frames[index] == null) {
      frames[index] = doGetFrameAt(index);
      frames[index].setController(this);
       
      if (index == 0) {
        frames[index].setLocation(100, 100);
      }

      frames[index].pack();
    }

    return frames[index];
  }


  /**
    * Resets all bits to 0. 
    */
  protected void clearBits() {  
    bits.and(new BitSet());
  }

  /**
    *
    */
  protected String getHelpMessage() {
    return getHelpMessage(whichFrame);
  }

  /**
    * called when user presses next button, loads the contents of the next
    * frame, hides the current frame and displays the next frame.
    */
  protected void showNext() {
    int nextFrameIndex = whichFrame + 1;
    if (whichFrame == 0 && !init) {                 
      LBLResourceFrame resFrame = (LBLResourceFrame) getFrameAt(0);

      if (resFrame.getBaseURL().startsWith("https")) {
        if (!initSSLInfo()) {
          return;
        }
      }
    }  

    try {
      if (!bits.get(nextFrameIndex)) {
        if (loadInputsIntoFrameAt(nextFrameIndex)) {
          bits.set(nextFrameIndex);
        }
      }

      hideAndShow(whichFrame, nextFrameIndex);
    } catch(RuntimeException ex) {
      ex.printStackTrace();
    } catch(Exception ex) {
      ex.printStackTrace();
      showDialog("Error", ex.getMessage());
    }
  }

  /**
    */
  protected GenCa[] getGenCas() {
    return info.getGenCas();
  }

  /**
    */
  protected void showDialog(String title, String info) {
    getFrameAt(whichFrame).showDialog(title, info);
  }

  /**
    * called when user presses prev button, hides the current frame and 
    * displays the previous frame.
    */
  protected void showPrevious() {
    hideAndShow(whichFrame, whichFrame-1);

    if (whichFrame == 0) {
      bits.clear(1);
    }
  }
    
  /**
    * called when user presses cancel button, hides the current frame and 
    * displays the first frame.
    */
  protected  void cancel() {
    info = null;
    clearBits();
    getFrameAt(0).clear();
    hideAndShow(whichFrame, 0);
  }

  /**
    * sets one frame visible and the other to invisible
    * The visible frame becomes the currentFrame.
    *
    * @param toHideIndex index of frame to hide
    * @param toShowIndex index of frame to show
    */
  protected void hideAndShow(int toHideIndex, int toShowIndex) {
    LBLFrame toHideFrame =  getFrameAt(toHideIndex);
    LBLFrame toShowFrame =  getFrameAt(toShowIndex);

    toHideFrame.setVisible(false);
    toShowFrame.setLocation(250, 250);
    toShowFrame.show();
    whichFrame = toShowIndex;
  }

  /**
    *
    */
  protected void signAndSave() {
     try {
       LBLSaveFrame saveFrame = (LBLSaveFrame) frames[frames.length - 1];
       P12Reader p12Reader = new P12Reader();
       CredentialInfo info = CredentialInfo.getInstance();

       if (info.isInitCredential()) {
         gen.setIssuer(
           new AkentiIssuer(
             new AkentiPrincipal(
               info.getSubjectDN(),
               info.getIssuerDN()
             )
           )
         );
       }
       else {
         p12Reader.readP12File(saveFrame.getKeyFileName(), saveFrame.getPassword());
         // XXX
         gen.setIssuer(
           new AkentiIssuer(
             new AkentiPrincipal(
               p12Reader.getSubjectDN(),
               p12Reader.getIssuerDN()
             )
           )
         );
       }

       AkentiCertificate cert = gen.generate();

       String hash = Util.createHashName(cert.getHashString());
       
       gen.setUniqueID(Util.createUniqueID(hash));

       java.util.Properties props = System.getProperties();

       props.setProperty("MD5withRSA", "RSA-MD5");

    
       if (info.isInitCredential()) {
         gen.sign("MD5withRSA", info.getPrivateKey());
       }
       else {
         gen.sign("MD5withRSA", p12Reader.getPrivateKey());
       }

       String fileName = saveFrame.getCertFileName();
        
       // XXX
       fileName += '.' + hash + ".";
       fileName = Util.createFileName(fileName);

       FileOutputStream out = new FileOutputStream(fileName);

       AkentiCertificateFactory.writeXMLCertToStream(cert, out);
       // AkentiCertificateFactory.writePEMCertToStream(cert, out);

       showDialog(
         "Info", 
         "Certificate " + fileName + " has been saved"
       );

       cancel();
    } catch(RuntimeException e) {
      e.printStackTrace();
    } catch(Exception e) {
      showDialog("Error", e.getMessage());
    }
  }

  protected void exit() {
    System.exit(1);
  }

  /**
    * Return whether or not initialized SSL info. 
    */
  protected boolean initSSLInfo() {
    if (!init) {
      init = startSSLPane(getFrameAt(0));
    }

    return init;
  }

  /**
    * Initializes SSL Parameters.
    */
  private void 
  initSSLInfo(String p12FileName, String passwordString, String casFileName) 
    throws IOException, GeneralSecurityException {
      char[] password = passwordString.toCharArray();
      KeyStore ks = KeyStore.getInstance("PKCS12");

      ks.load(new FileInputStream(p12FileName), password);

      X509CertReader xcr = new X509CertReader();
      X509Certificate[] caCerts = xcr.readX509Certs(casFileName);

      for (int i = 0; i < caCerts.length; ++i) {
        String alias = caCerts[i].getSubjectDN().toString();

        ks.setCertificateEntry(alias, caCerts[i]);
      }

      KeyManagerFactory kfac = KeyManagerFactory.getInstance("SunX509");

      kfac.init(ks, password);

      TrustManagerFactory tfac = TrustManagerFactory.getInstance("SunX509");

      tfac.init(ks);

      SSLContext ctx = SSLContext.getInstance("SSLv3");
      File f = new File(".rand");
      byte seed[] = null;

      try {
        if (f.exists()) {
          FileInputStream fis = new FileInputStream(f);
          byte temp[] = new byte[20];

          fis.read(temp);
          seed = temp;
        }
      } catch(IOException ex) {
        // DO NOTHING
      }

      SecureRandom random = SecureRandom.getInstance("SHA1PRNG");

      if (seed == null) {
        System.out.println("this will take some time...");
        seed = random.generateSeed(20);

        try {
          FileOutputStream fos = new FileOutputStream(f);
          fos.write(seed);
          fos.flush();
          fos.close();

          System.out.println("A .rand file is saved into this directory");

        } catch(IOException ex) {
        }
      } else {
        random.setSeed(seed);
      }

      ctx.init(
        kfac.getKeyManagers(),
        tfac.getTrustManagers(),
        random 
      );

      SSLSocketFactory sslFactory = ctx.getSocketFactory();
      HttpsURLConnection.setDefaultSSLSocketFactory(sslFactory);

      HttpsURLConnection.setDefaultHostnameVerifier(new com.sun.net.ssl.HostnameVerifier() {
        public boolean verify (String urlHostname, String string) {
          return true;
        }
      });
  }

  /**
    * Starts a pane asking the user for the pkcs12 file, passphrase, and 
    * trusted CA file.
    */ 
  private boolean startSSLPane(LBLFrame parent) {
    // MOVE THIS HEADER
    String HEADER =                       
      "<html><font size=+1 color=blue> SSL Info" +
      "<li><font color=blue>Enter the name and password of the pkcs12 file.<i>" +
      "<li><font color=blue>Enter the name of the trusted CA(S) file."
      ; 

    LBLSSLPane pane = new LBLSSLPane(HEADER);

    pane.setPkcs12FileName(System.getProperty("ClientPkcs12File", ""));
    pane.setCAFileName(System.getProperty("CertificateAuthorityFile", ""));

    // setFunctions
    JDialog dialog = pane.createDialog(parent, "SSL Info");

    do {
      dialog.show();

      if (pane.getValue().equals("CANCEL")) {
        break;
      }

      try {
        SSLInfo.initSSLInfo(
          pane.getPkcs12FileName(),
          pane.getPassword(),
          pane.getCAFileName()
        );
  
        CredentialInfo info = CredentialInfo.getInstance();
 
        info.readP12File(pane.getPkcs12FileName(), pane.getPassword());
        return true;
      } catch(IOException ex) {
        LBLDialogFactory.showInfoDialog(parent, ex.getMessage());
      } catch(GeneralSecurityException ex) {
        LBLDialogFactory.showInfoDialog(parent, ex.getMessage());
      }
    } while (true);
    
    // MEANS PERSON CANCEELED
    return false;
  }
}

