package lbl.akenti.gen.controller.attr;

import java.net.URL;
import java.security.Security;
import java.util.Properties;

import lbl.akenti.gen.controller.LBLFrame;
import lbl.akenti.gen.controller.LBLResourceFrame;
import lbl.akenti.gen.controller.LBLIssuerFrame;
import lbl.akenti.gen.controller.LBLTimeFrame;
import lbl.akenti.gen.controller.LBLCacheFrame;
import lbl.akenti.gen.controller.LBLReviewFrame;
import lbl.akenti.gen.controller.LBLSaveFrame;
import lbl.akenti.gen.controller.AbstractController;

import lbl.akenti.util.UTCTime;

import lbl.akenti.gen.common.GenInfo;
import lbl.akenti.gen.common.GenAttribute;
import lbl.akenti.gen.common.GenValue;

import lbl.akenti.certs.AttributeCertGenerator;
import lbl.akenti.certs.elements.AkentiIssuer;
import lbl.akenti.certs.elements.AkentiPrincipal;



/**
  * Generates an attribute certificate.
  * 
  * @see AbstractController
  */
public class AtController extends AbstractController implements AtConsts {

  /**
    * Constructor.
    *
    * @see AbstractController#doGetFrameAt(int)
    */
  public AtController() {
    super(7);     // We will be using 7 frames.
  }

  /**
    * Shows the first frame.
    *
    * @see AbstractController#start()
    */
  public void start() {
    super.start();
  }

  /**
    * Creates the frame associated with the given index.
    *
    * @param index of the frame to be created
    * @return an LBLFrame that corresponds to index.
    *
    * @see AbstractController#getFrameAt(index)
    */
  protected LBLFrame doGetFrameAt(int i) {
    switch(i) {
      case RESOURCE_INDEX:
        return new LBLResourceFrame(TITLES[i], HEADERS[i]);
      case ATTRIBUTE_INDEX:
        return new AtValueFrame(TITLES[i], HEADERS[i]);
      case ISSUER_INDEX:
        return new LBLIssuerFrame(TITLES[i], HEADERS[i]);
      case USER_INDEX: 
        return new AtUserFrame(TITLES[i], HEADERS[i]);
      case TIME_INDEX:
        return new LBLTimeFrame(TITLES[i], HEADERS[i]);
      case REVIEW_INDEX:
        return new LBLReviewFrame(TITLES[i], HEADERS[i]);
      case SIGN_INDEX:
        return new LBLSaveFrame(TITLES[i], HEADERS[i]);
      default:
        throw new RuntimeException();
    }
  }

  /**
    * Loads inputs in the frame that corresponds to <i>index</i>.
    *
    * @param index index of frame that needs to be loaded
    *
    * @return true if this method should be called when displaying this
    *              frame in the future.
    *         false otherwise.
    *
    * @see AbstractController#showNext()
    * @see AbstractController#loadInputsIntoFrameAt(int)
    * @see AbstractController#clearBits()
    */
  protected boolean loadInputsIntoFrameAt(int index) throws Exception{
    switch(index) {
      case ATTRIBUTE_INDEX:
	LBLResourceFrame resourceFrame = (LBLResourceFrame) getFrameAt(RESOURCE_INDEX);
        URL base = new URL(resourceFrame.getBaseURL());
        String resource = resourceFrame.getResource();
	      
        if (info == null || !(info.hasResource(resource) && info.hasBaseURL(base))) {
          // Make sure we load the GenInfo first.
          GenInfo tempInfo = new GenInfo(resource, base);

	  tempInfo.loadResFileInfo();
	  tempInfo.loadPolicies();

          if (!tempInfo.getTopResource().equals(resource)) {
            System.out.println("*************");
            System.out.println(tempInfo.getTopResource());
            System.out.println(resource);
            System.out.println("*************");
            String msg = "<html>Found wrong policy in " + resource +   " directory <BR>";

            msg += "Policy describes " + tempInfo.getTopResource() + "</html>";
            throw new Exception(msg);
          }

          this.info = tempInfo;

	  GenAttribute[] attributes = info.getGenAttributes();
	  AtValueFrame valFrame = (AtValueFrame) getFrameAt(index);

	  valFrame.clear();

	  for (int i = 0; i< attributes.length; i++) {
	    if (!attributes[i].isX509Attribute()) {
	      valFrame.addAttribute(attributes[i]);
            }
          }

          clearBits();
        }

        return true;
      case ISSUER_INDEX: 
        LBLIssuerFrame issuerFrame = (LBLIssuerFrame) getFrameAt(index);
        AtValueFrame valFrame = (AtValueFrame)getFrameAt(ATTRIBUTE_INDEX);
        GenValue value = valFrame.getGenValue();

        issuerFrame.setOptional(true);
	issuerFrame.setIssuers(value.getAkentiPrincipals());
	return false;
      case USER_INDEX:
        return true;
      case TIME_INDEX:
        return true;
      case REVIEW_INDEX:
        AttributeCertGenerator gen = new AttributeCertGenerator();

        gen.setVersionNumber(2);
        gen.setUniqueID("UID");

        LBLTimeFrame timeFrame = (LBLTimeFrame) getFrameAt(TIME_INDEX);
        UTCTime utc = new UTCTime();

        gen.setNotBefore(utc);
        gen.setNotAfter(new UTCTime(utc, timeFrame.getLifeTime()));

        issuerFrame = (LBLIssuerFrame) getFrameAt(ISSUER_INDEX);
        
        AkentiPrincipal issuer = issuerFrame.getIssuer();

        if (issuer != null) {
          gen.setIssuer(new AkentiIssuer(issuer));
        }

        valFrame = (AtValueFrame) getFrameAt(ATTRIBUTE_INDEX);

        gen.setAttribute(valFrame.getAttributeName());
        gen.setValue(valFrame.getValueName());

        AtUserFrame userFrame = (AtUserFrame) getFrameAt(USER_INDEX);

        gen.setPrincipal(userFrame.getAkentiPrincipal());

	LBLReviewFrame reviewFrame = (LBLReviewFrame) getFrameAt(index);

	reviewFrame.setCertificate(gen.generate());
        this.gen = gen;
	return false;
      case SIGN_INDEX:
        LBLSaveFrame signFrame = (LBLSaveFrame) getFrameAt(index);

        signFrame.clear();
        return false;
      default:
        throw new RuntimeException();
    }
  }

  /**
    */
  protected String getHelpMessage(int index) {
    return helpMess[index];
  }

  /**
    * Reads Confi file, sets provider and finally starts 
    * the controller. 
    *
    */
  public static void main (String[] args) {
    try {
      // Load Properties.
      if (args.length == 1) {
        Properties props = System.getProperties();

        props.load(new java.io.FileInputStream(args[0]));
        System.out.println(props);
      }

      Security.addProvider(new com.sun.net.ssl.internal.ssl.Provider());

      //String d = "java.protocol.handler.pkgs";
      //Properties props = System.getProperties();

      //props.put("java.protocol.handler.pkgs", "com.sun.net.ssl.internal.www.protocol");

      new AtController().start();
    } catch(Exception e) {
      System.out.println(e.getMessage());
    }
  }
}
