package lbl.akenti.messaging;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.Writer;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import javax.xml.bind.JAXBException;
import lbl.akenti.jaxb.AkentiResponseType;
import lbl.akenti.jaxb.*;
import lbl.akenti.jaxb.CheckAccessResponseType;
import lbl.akenti.jaxb.CheckPolicyResponseType;
import lbl.akenti.jaxb.CreateResourceResponseType;
import lbl.akenti.jaxb.DeleteResourceResponseType;
import lbl.akenti.jaxb.GetResourceInfoResponseType;
import lbl.akenti.jaxb.ResourceBrowseResponseType;
import lbl.akenti.jaxb.UploadPolicyCertificateResponseType;
import org.w3c.dom.Element;

/**
 * Represents an immutable non-empty list of <code>AkentiResponse</code>s that
 * are created by the Akenti server.                            <BR>
 *
 * Currently The server handles the following responses:        <BR>
 *
 * CheckAccessResponse:                                         <BR>
 *   Used when determining a user's access to a resource.       <BR>
 *
 * CheckPolicyResponse:                                         <BR>
 *   Used by stakholders to debug policy                        <BR>
 *
 * ResourceBrowseResponse:                                      <BR>
 *   Used to determine daughter resources of given resource     <BR>
 *
 * CreateResourceResponse:                                      <BR>
 *   Used to create resources                                   <BR>
 *
 * UploadPolicyCertificateResponse:                             <BR>
 *   Used to Upload a policy certificate                        <BR>
 *
 * GetResoourceInfoResponse:                                    <BR>
 *   Used to upload a resource attrs/hints                      <BR>
 *
 * Each response contains the client-generated unique id of the request
 * it is responding to. That can be useful when clients are sending or
 * keeping track of multiple requests.
 */
final public
class AkentiResponses {
  private List responses;
  
  /**
   * Constructs a simple <code>AkentiResponses</code> object containing a single AkentiResponse.
   * @param ar a AkentiResponse
   */
  public AkentiResponses(AkentiResponse ar) {
    this(new AkentiResponse[]{ar});
  }
  
  /**
   * Constructs a simple <code>AkentiResponses</code> object containing the
   * <code>AkentiResponse</code>s listed in the array.
   * @param ars an array of AkentiResponse objects
   */
  public AkentiResponses(AkentiResponse[] ars) {
    responses = new ArrayList(ars.length);
    responses.add(ars[0]);
    
    for (int i = 1; i < ars.length; ++i) {
      responses.add(ars[i]);
    }
    
    responses = Collections.unmodifiableList(responses);
  }
  
  /**
   * Constructs this <code>AkentiResponses</code> from an input stream
   * containing <code>XML</code> data.
   * @param in an InputStream constructed with a streamized Responses
   * @throws IOException when unable to read this object
   * @throws ParseException if the stream does not define a valid Responses
   */
  public AkentiResponses(InputStream in) throws IOException, ParseException {
    try {
      init((AkentiResponsesGenerated) Util.getUnmarshaller().unmarshal(in));
    } catch(JAXBException ex) {
      throw new IOException(ex.getMessage());
    }
  }
  
  /**
   * Constructs this <code>AkentiResponses</code> from a <code>DOM</code> element.
   * @param element an element containing an AkentiResponses
   * @throws ParseException if the stream does not define a valid AkentiResponses
   * @throws IOException when unable to build this object
   */
  public AkentiResponses(Element element) throws IOException, ParseException {
    try {
      init((AkentiResponsesGenerated) Util.getUnmarshaller().unmarshal(element));
    } catch(JAXBException ex) {
      throw new ParseException(ex.getMessage(), -1);
    }
  }
  
  /**
   * Returns the responses as an iterator.
   * @see AkentiResponse
   * @return the iterator
   */
  public Iterator getAkentiResponses() {
    return responses.listIterator();
  }
  
  /**
   * Returns the responses as an array.
   * @see AkentiResponse
   * @return the request array
   */
  public AkentiResponse[] toAkentiResponsesArray() {
    AkentiResponse[] ret = new AkentiResponse[responses.size()];
    
    responses.toArray(ret);
    return ret;
  }
  
  /**
   * Turns this response into a stream. To reconstitute the response
   * use the constructor that takes an InputStream.
   * @param out the stream
   * @throws IOException if the responses can't be streamed
   */
  public void toStream(OutputStream out) throws IOException {
    try {
      Util.getMarshaller().marshal(toJAXBImpl(), out);
    } catch(JAXBException ex) {
      throw new IOException(ex.getMessage());
    }
  }
  
  /** Marshals this responses's data into the given writer.
   * @param writer the writer
   * @throws IOException if there is an problem marshalling the data
   */
  public void toWriter(Writer writer) throws IOException {
    try {
      Util.getMarshaller().marshal(toJAXBImpl(), writer);
    } catch(JAXBException ex) {
      throw new IOException(ex.getMessage());
    }
  }
  
  /** Marshals this request's data into the given element.
   * @param element the element which will contain the data.
   * @throws ParseException if there is an problem marshalling the data
   */
  public void toElement(Element element) throws ParseException {
    try {
      Util.getMarshaller().marshal(toJAXBImpl(), element);
    } catch(JAXBException ex) {
      throw new ParseException(ex.getMessage(), -1);
    }
  }
  
  /**
   */
  private void init(AkentiResponsesGenerated impl) throws IOException, ParseException {
    List temp = impl.getAkentiResponse();
    Iterator iter = temp.listIterator();
    
    responses = new ArrayList(temp.size());
    
    do {
      AkentiResponseType resp = (AkentiResponseType) iter.next();
      
      if (resp instanceof CheckAccessResponseType) {
        responses.add(new CheckAccessResponse((CheckAccessResponseType) resp));
      } else if (resp instanceof CheckPolicyResponseType) {
        responses.add(new CheckPolicyResponse((CheckPolicyResponseType) resp));
      } else if (resp instanceof ResourceBrowseResponseType) {
        responses.add(new ResourceBrowseResponse((ResourceBrowseResponseType) resp));
      } else if (resp instanceof DeleteResourceResponseType) {
        responses.add(new DeleteResourceResponse((DeleteResourceResponseType) resp));
      } else if (resp instanceof CreateResourceResponseType) {
        responses.add(new CreateResourceResponse((CreateResourceResponseType) resp));
      } else if (resp instanceof UploadPolicyCertificateResponseType) {
        responses.add(new UploadPolicyCertificateResponse((UploadPolicyCertificateResponseType) resp));
      } else if (resp instanceof DeletePolicyCertificateResponseType) {
        responses.add(new DeletePolicyCertificateResponse((DeletePolicyCertificateResponseType) resp));
      } else if (resp instanceof GetResourceInfoResponseType) {
        responses.add(new GetResourceInfoResponse((GetResourceInfoResponseType) resp));
      } else {
        throw new RuntimeException();
      }
    } while (iter.hasNext());
    
    responses = Collections.unmodifiableList(responses);
  }
  
  /**
   * FOR XML STUFF
   */
  private AkentiResponsesGenerated toJAXBImpl() throws JAXBException {
    AkentiResponsesGenerated impl = Util.getObjectFactory().createAkentiResponsesGenerated();
    Iterator iter = getAkentiResponses();
    List temp = impl.getAkentiResponse();
    
    do {
      AkentiResponse ar = (AkentiResponse) iter.next();
      
      temp.add(ar.toJAXBImpl());
    } while (iter.hasNext());
    
    return impl;
  }
}
