/*
 * CertInfo.java
 *
 * Created on July 28, 2004, 9:21 AM
 */

package lbl.akenti.policyEngine.check;

import java.util.ArrayList;
import java.util.List;
import lbl.akenti.certs.AkentiCertificate;
import lbl.akenti.messaging.AkentiMessage;

/**
 * This is a psudeo-extension of the CertificateInfo class. Basically,
 * it does everything CertificiateInfo does but holds some special info
 * for CheckPolicy.
 * @author Tommaso Sciortino
 * @see lbl.akenti.messaging.CertificateInfo
 */
public class CertInfo {
  private AkentiCertificate cert;
  private List mesgsList;
  private String location;
  private String hash;
  private String resource;
  private boolean used;  
  private boolean isError;
  private boolean isLeaf;
    
  /**
   * Constructs this <code>CertInfo</code> with messages.
   * @param cert the certificate being described
   * @param mesgs AkentiMessages describing the certs status
   * @param location the local location of the certificate
   * @param used was this certificate used to check the users access
   */
  public CertInfo(AkentiCertificate cert, AkentiMessage[] mesgs, String location, boolean used) {
    if (cert == null || mesgs == null || location == null) {
      throw new IllegalArgumentException();
    }
    
    this.cert = cert;
    this.location = location;
    this.used = used;
    
    mesgsList = new ArrayList(mesgs.length);
    for (int i = 0; i < mesgs.length; ++i) {
      mesgsList.add(mesgs[i]);
    }
  
    this.isLeaf = false;
    this.isError = false;
    this.resource = "NO_RESOURCE";
  }
  
  /**
   * Constructs this <code>CertInfo</code> with a single message.
   * @param cert the certificate being described
   * @param mesg AkentiMessage describing the certs status
   * @param location the local location of the certificate
   * @param used was this certificate used to check the users access
   */
  public CertInfo(AkentiCertificate cert, AkentiMessage mesg, String location, boolean used) {
    this(cert,new AkentiMessage[] {mesg},location, used);
  }
  
  /**
   * Constructs this <code>CertInfo</code> without a location.
   * @param cert the certificate being described
   * @param mesgs AkentiMessages describing the certs status
   * @param used was this certificate used to check the users access
   */
  public CertInfo(AkentiCertificate cert, AkentiMessage[] mesgs, boolean used) {
    this(cert,mesgs,"",used);  
  }
  
  /**
   * Constructs this <code>CertInfo</code> without a location and one AkentiMessage.
   * @param cert the certificate being described
   * @param mesg AkentiMessage describing the certs status
   * @param used was this certificate used to check the users access
   */
  public CertInfo(AkentiCertificate cert, AkentiMessage mesg, boolean used) {
    this(cert,mesg,"",used);
  }
  
  /**
   * Returns the <code>AkentiCertificate</code>.
   * @return the certificate being described
   */
  public AkentiCertificate getAkentiCertificate() {
    return cert;
  }
  
  /**
   * Returns the used attribute.
   * @return true if this certificate was used
   */
  public boolean isUsed() {
    return used;
  }

  /**
   * Returns the location.
   * @return the local location of the certificate
   */
  public String getLocation() {
    return location;
  }

  /**
   * Sets the used attribute.
   * @param used whether the certificate was used
   */
  public void setUsed(boolean used) {
    this.used = used;
  }

  /**
   * Sets the location.
   * @param location the local location of the certificate
   */
  public void setLocation(String location) {
    this.location = location;
  }
  
  /**
   * Returns <i>true</i> if at least one <code>AkentiMessage</code> is present.
   * @return true if this has a certificate
   */
  public boolean hasAkentiMessages() {
    return (mesgsList != null || mesgsList.size() != 0);
  }
    
  /**
   * Adds an <code>AkentiMessage</code>.
   * @param mesg a message describing this certificate
   */
  public void addAkentiMessage(AkentiMessage mesg) {
    mesgsList.add(mesg);
  }
  
  /**
   * Returns the <code>AkentiMessage</code>s.
   * @return the messages describing this certificate
   */
  public AkentiMessage[] getAkentiMessages() {
    return (AkentiMessage[]) mesgsList.toArray(new AkentiMessage[mesgsList.size()]);
  }
  
  
  /**
   * Sets whether there was an error in checking the policy.
   * @param isError whether there was an error
   */  
  public void setError(boolean isError) {
    this.isError = isError;
  }
  
  /**
   * True if there was an error in checking the policy.
   * @return true if there was an error
   */  
  public boolean isError() {
    return isError;
  }
  
  /**
   * Sets whether this certificate is a new leaf or subserviant to the previous one.
   * @param isLeaf whether the certificate is leaf
   */  
  public void setLeaf(boolean isLeaf) {
    this.isLeaf = isLeaf;
  }
  
  /**
   * Returns whether this certificate is a new leaf or subserviant to the previous one.
   * @return trueif the certificate is leaf
   */  
  public boolean isLeaf() {
    return isLeaf;
  }
  
  /**
   * Sets the hash code coresponding to this certificate
   * @param hash a hash describing the certificate
   */  
  public void setHashCode(String hash) {
    this.hash = hash;
  }
  
  /**
   * Returns the hash code coresponding to this certificate
   * @return a hash describing the certificate
   */  
  public String getHashCode() {
    return hash;
  }

  /**
   * The type of certificate being held (either Policy, UseCondition, or dummy).
   * @return the type of the certificate
   */  
  public String getType() {
    return getAkentiCertificate().getType();
  }
  
  /**
   * Sets the resource the certificate describes
   * @param resource the certificate's resource
   */  
  public void setResource(String resource) {
    this.resource = resource;
  }
  
  /**
   * Returns the resource the certificate describes
   * @return the certificate's resource
   */  
  public String getResource() {
    return resource;
  }
  
}