package lbl.akenti.policyEngine.impl; 

import java.util.ArrayList;
import java.util.List;
import lbl.akenti.certs.Certificate;
import lbl.akenti.certs.elements.AttributeInfo;
import lbl.akenti.policyEngine.CertificateCache;
import lbl.akenti.policyEngine.PolicyContext;
import lbl.akenti.policyEngine.Runtime;
import lbl.akenti.policyEngine.ExternalEvaluator;
import lbl.akenti.policyEngine.TrustVerifier;

public class AkentiExternalEvaluator implements ExternalEvaluator {
  private Runtime runtime;
  private int cacheInterval;
  private List certs = new ArrayList();

  protected AkentiExternalEvaluator() {
  }

  /**
    * Returns <i>true</i> if this evaluator knows how to 
    * handle the <code>SYSTEM attribute</code7>.
    *
    * @see evaluateSystemAttribute(int, const AttributeInfo&)
    */
  public boolean canHandleSystemAttribute(String attribute) {
    return false;
  }

  /**
    * Returns <i>true</i> if (attrInfo.getAttribute() op  attrInfo.getValue())
    *
    * Note: the engine will call this method only if 
    *      canHandleSystemAttribute<attrInfo.getAttribute() returned true.
    *
    * @see AttributeInfo 
    * @see int
    */
  public boolean evaluateSystemAttribute(int op, AttributeInfo attrInfo) {
    throw new UnsupportedOperationException();
  }

  /**
    * Returns <i>true</i> if this evaluator knows how to 
    * handle the <code>EXT_AUTH attribute</code>.
    *
    * @see evaluateExternalAttribute(int, const AttributeInfo&)
    */
  public boolean canHandleExternalAttribute(String attribute) {
    return false;
  }

  /**
    * Returns <i>true</i> if (attrInfo.getAttribute() op  attrInfo.getValue())
    *
    * Note: the engine will call this method only if 
    *        canHandleHandleExternalAttribute<attrInfo.getAttribute()> returned true.
    *
    * @see AttributeInfo 
    */
  public boolean evaluateExternalAttribute(int op, AttributeInfo attrInfo) {
    throw new UnsupportedOperationException();
  }

  /**
    * Returns the cache Interval.
    *   If return value > 0
    *     The engine may not cache the capability certificate
    *     but if it does it will cache it for no longer than the cache Interval. 
    * 
    *   Note: a return value < 0 implies the external evaluator does not care.
    */
  public final int getCacheInterval() {
    return cacheInterval;
  }

  public final Certificate[] getCollectedCertificates() {
    return (Certificate[]) certs.toArray(new Certificate[0]);
  }

  /**
    * Allows the external evaluator to suggest whether it likes 
    * the capability certificate to be cached. A negative value
    * implies the evaluator does not care.
    */
  protected void setCacheInterval(int numOfseconds) {
    this.cacheInterval = cacheInterval;
  }

  /**
    * External evaluators should use this method to store certificates
    * they have collected and used in making their decisions. These
    * certificates will eventually end up in the PolicyContext
    * when appropriate.
    */
  protected void addCollectedCertificate(Certificate cert) {
    certs.add(cert);
  }

  /**
    * @see PolicyContext
    */
  protected PolicyContext getPolicyContext() {
    return getRuntime().getPolicyContext(); 
  }

  /**
    * @see TrustVerifier 
    */
  protected TrustVerifier getTrustVerifier() {
    return getRuntime().getTrustVerifier(); 
  }

  /**
    *
    * @see CertificateCache 
    */
  protected CertificateCache getCertificateCache() {
    return getRuntime().getCertificateCache(); 
  }

  /**
    */
  private Runtime getRuntime() {
    return runtime;
  }

  /**
    * @see Runtime
    */
  void setRuntime(Runtime runtime) {
    this.runtime = runtime;
  }
}
