package lbl.akenti.policyManager.base;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.net.URL;
import java.security.GeneralSecurityException;
import java.text.ParseException;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Vector;
import lbl.akenti.certs.PolicyCertificate;
import lbl.akenti.certs.elements.AkentiPrincipal;
import lbl.akenti.certs.elements.CertificateAuthorityInfo;
import lbl.akenti.certs.elements.Directory;
import lbl.akenti.certs.elements.DistinguishedName;
import lbl.akenti.certs.elements.UseCondIssuerGroup;
import lbl.akenti.messaging.CertificateInfo;
import lbl.akenti.messaging.CheckPolicyRequest;
import lbl.akenti.messaging.CheckPolicyResponse;
import lbl.akenti.messaging.GetResourceInfoRequest;
import lbl.akenti.messaging.GetResourceInfoResponse;
import lbl.akenti.messaging.PolicyInfo;
import lbl.akenti.protocolHandler.LocalProtocolHandler;
import lbl.akenti.protocolHandler.ProtocolHandler;
import lbl.akenti.protocolHandler.RemoteProtocolHandler;
import lbl.akenti.util.AkentiConstants;
import org.xml.sax.SAXException;

/**
 */
public class GenInfo implements AkentiConstants {
  private URL baseURL;             // base pointer to the root resource
  
  private String resource;         // name of the resource
  
  private ResFileInfo resFileInfo; // hints from the resfile
  
  private GenPolCert polCert;      // chain of policy certificates
  
  public GenInfo(String resource, URL baseURL) {
    this.resource = resource;
    this.baseURL  = baseURL;
    resFileInfo = new ResFileInfo();
    polCert = new GenPolCert();
  }
  
  /**
   */
  public boolean isRoot() {
    return resource.indexOf("/") == -1;
  }
  
  /**
   */
  public String getResource() {
    return resource;
  }
  
  /**
   */
  public URL getBaseURL() {
    return baseURL;
  }
  
  /**
   */
  public String getTopResource() {
    return polCert.getResource();
  }
  
  public boolean hasResource(String resource) {
    return this.resource.equals(resource);
  }
  
  public boolean hasBaseURL(URL url) {
    return url.equals(baseURL);
  }
  
  public GenAttribute[] getX509GenAttributes() {
    return resFileInfo.getX509GenAttributes();
  }
  
  public boolean containsX509GenAttributes() {
    return resFileInfo.containsX509GenAttributes();
  }
  
  public GenAttribute[] getGenAttributes() {
    return resFileInfo.getGenAttributes();
  }
  
  public GenAttribute getGenAttribute(String attrName) {
    return resFileInfo.getGenAttribute(attrName);
  }
  
  /**
   * Combines the cas and their id dirs from
   * policy and resfile.
   */
  public GenCa[] getGenCas() {
    HashMap cas = new HashMap();
    GenCa[] genCas = resFileInfo.getGenCas();
    
    for (int i = 0; i < genCas.length; ++i) {
      cas.put(genCas[i].getDistinguishedName(), genCas[i]);
    }
    
    CertificateAuthorityInfo[] caInfos = polCert.getCertificateAuthorityInfos();
    
    for (int i = 0; i < caInfos.length; ++i) {
      DistinguishedName key = caInfos[i].getDistinguishedName();
      GenCa genCa = (GenCa) cas.get(key);
      
      if (genCa == null) {
        genCa = new GenCa(key);
      }
      
      genCa.addDirs(caInfos[i].getIdentityDirectories());
      cas.put(key, genCa);
    }
    
    Collection col = cas.values();
    
    genCas = new GenCa[col.size()];
    col.toArray(genCas);
    return genCas;
  }
  
  public String[] getActions() {
    return resFileInfo.getActions();
  }
  
  public CertificateAuthorityInfo[]
  getCertificateAuthorityInfos() {
    return polCert.getCertificateAuthorityInfos();
  }
  
  public UseCondIssuerGroup[]
  getUseCondIssuerGroups() {
    return polCert.getUseCondIssuerGroups();
  }
  
  public AkentiPrincipal[]
  getUCIssuers() {
    UseCondIssuerGroup[] ucGroups = getUseCondIssuerGroups();
    HashSet ucIssuers = new HashSet();
    
    for (int i = 0; i < ucGroups.length; ++i) {
      AkentiPrincipal[] aps = ucGroups[i].getUCIssuers();
      
      for (int j = 0; j < aps.length; ++j) {
        ucIssuers.add(aps[j]);
      }
    }
    
    AkentiPrincipal []temp = new AkentiPrincipal[ucIssuers.size()];
    
    ucIssuers.toArray(temp);
    return temp;
  }
  
  public Directory[]
  getAttributeDirectories() {
    return polCert.getAttributeDirectories();
  }
  
  public long getCacheTime() {
    return polCert.getCacheTime();
  }
  
  public String paramString() {
    StringBuffer sb = new StringBuffer();
    
    sb.append('[');
    sb.append("Base");
    sb.append('=');
    sb.append(baseURL);
    sb.append(',');
    sb.append(resFileInfo.paramString());
    sb.append(',');
    sb.append(polCert.paramString());
    sb.append('=');
    return sb.toString();
  }
  
  /**
   * Loads the first encountered ResAttrFile.
   *
   * @exception ParseException if a resfile is
   *            found but is not well formatted.
   *
   * @return true if one such file is found
   *         false otherwise
   *
   * @see ResFileInfo
   * @see ResFileParser
   */
  public boolean loadResFileInfo() throws ParseException, IOException {
    resFileInfo = new ResFileInfo();
    
    ProtocolHandler eng = null;
    if (baseURL.getProtocol().equals("file")) {
      eng = new LocalProtocolHandler();
    } else {
      eng = new RemoteProtocolHandler(baseURL);
    }
    
    GetResourceInfoRequest req = new GetResourceInfoRequest(ProtocolHandler.getID(), resource);
    GetResourceInfoResponse resp = eng.process(req);
    String resourceInfo = resp.getResourceInfo();
    
    if (resourceInfo == null || resourceInfo.length() == 0) {
      return false;
    }
    
    try {
      ResFileParser parser = new ResFileParser(new ByteArrayInputStream(resourceInfo.getBytes()));
      parser.loadResFileInfo(resFileInfo);
      
    } catch(SAXException ex) {
      throw new ParseException(ex.getMessage(), -1);
    }
    
    return true;
    
  }
  
  /**
   * Loads all the policy certificates and forms the
   * chain.
   *
   * @return true if the root policy was loaded.
   *         false if the root policy was not loaded.
   *
   * @exception ParseException
   *            if any of the policy certificates is badly formed.
   *            or if this is not a root resource and the root
   *            policy was not found.
   *
   * @see GenPolCert
   * @see lbl.akenti.certs.PolicyCertificate
   * @see lbl.akenti.certs.AkentiCertificateFactory
   *
   */
  public boolean loadPolicies() throws IOException, ParseException, GeneralSecurityException  {
    Vector policyCerts = new Vector();
    
    polCert = new GenPolCert();
    
    ProtocolHandler eng = null;
    if (baseURL.getProtocol().equals("file")) {
      eng = new LocalProtocolHandler();
    } else {
      eng = new RemoteProtocolHandler(baseURL);
    }
    
    CheckPolicyRequest req = new CheckPolicyRequest(ProtocolHandler.getID(), resource);
    CheckPolicyResponse resp = eng.process(req);
    Iterator iter = resp.getPolicyInfos();
    
    if (!iter.hasNext()) {
      throw new GeneralSecurityException(resp.getMessage().toString());
    }
    
    while (iter.hasNext()) {
      PolicyInfo pi = (PolicyInfo) iter.next();
      CertificateInfo ci = pi.toCertificateInfosArray()[0];
      PolicyCertificate cert = (PolicyCertificate) ci.getAkentiCertificate();
      
      policyCerts.add(cert);
    }
    
    int last = policyCerts.size() - 1;
    
    if (last != -1) {
      // Build the chain.
      polCert = new GenPolCert(null, (PolicyCertificate) policyCerts.elementAt(last));
      
      for (int i = last - 1; i > -1; --i) {
        polCert = new GenPolCert(polCert, (PolicyCertificate) policyCerts.elementAt(i));
      }
      
      return true;
    }
    
    return false;
  }
}
