package lbl.akenti.policyManager.base;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.ParseException;
import java.util.HashSet;
import lbl.akenti.certs.elements.AkentiPrincipal;
import lbl.akenti.certs.elements.Directory;
import lbl.akenti.certs.elements.DistinguishedName;
import lbl.akenti.util.ParsingException;
import lbl.akenti.util.XMLUtil;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/*
 * PERMISSIVE PARSING
 */
public class ResFileParser {
  /**
   * A public constant to control debugging level;
   */
  public static final boolean DEBUG = false;
  private Document tree;
  
  /**
   * Constructor
   * Converts the String to a InputStreamReader and calls
   * this(InputStreamReader).
   *
   * @param filename name of file to be parsed
   */
  public ResFileParser(String filename)
  throws IOException, FileNotFoundException, SAXException, ParseException {
    this(new FileInputStream(filename));
  }
  
  /**
   * Constructor
   * Converts the URL to an InputStreamReader and calls
   * this(InputStreamReader).
   *
   * @param url of the file to be parsed
   */
  public ResFileParser(URL url)
  throws IOException, SAXException, ParseException {
    this(url.openStream());
  }
  
  /**
   * Constructor
   * Parses the file using DOM parser, removes the newlines
   * and stores it in org.w3c.dom.Document tree
   *
   * @param in of the file to be parsed
   */
  public ResFileParser(InputStream in)
  throws IOException, SAXException, ParseException {
    Element root = XMLUtil.parseDocument(in).getDocumentElement();
    
    if (root == null) {
      throw new ParsingException("No root element");
    }
    
    XMLUtil.filter(root);
    tree = ((Node)root).getOwnerDocument();
  }
  
  /**
   * Parses the entire resInfo file and adds GenActions and GenAttributes to
   * the ResFileInfo.
   *
   * @param resInfo resInfo to add Actions and Attributes to.
   */
  public void loadResFileInfo(ResFileInfo resInfo) throws ParsingException, MalformedURLException {
    NodeList allCAs = tree.getElementsByTagName("ca");
    boolean caExists = false;
    
    if(allCAs.getLength() <= 0) {
      throw new ParsingException("resattrs.xml must have at least one CA");
    }
    
    for(int i = 0; i < allCAs.getLength(); i++) {
      if(allCAs.item(i).hasAttributes()) {
        caExists = true;
        GenAttribute []attributes = readX509Attributes(allCAs.item(i));
        
        for (int j = 0; j < attributes.length; j++) {
          resInfo.addGenAttribute(attributes[j]);
          Object value = attributes[j].getValues()[0];
          
          GenX509Value val = (GenX509Value) value;
          resInfo.addGenCas(val.getGenCas());
        }
      }
      
      if(i == allCAs.getLength()-1 && !caExists){
        throw new ParsingException("resattrs.xml must have at least one CA");
      }
    }
    
    NodeList allAttrs = tree.getElementsByTagName("attr");
    
    if(allAttrs.getLength() <= 0) {
      throw new ParsingException("resattrs.xml must have at least one Attribute");
    }
    
    for(int k = 0; k < allAttrs.getLength(); k++) {
      resInfo.addGenAttribute(readAttr(allAttrs.item(k)));
    }
    
    GenActions actions = readActions();
    resInfo.addActions(actions.getActions());
    
    if (DEBUG) {
      System.err.println("$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$");
      System.err.println(resInfo);
      System.err.println("$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$");
    }
  }
  
  /**
   * Read in group of X509 Attributes.
   *
   * @return GenAttribute[] the array of GenAttributes read in.
   */
  private GenAttribute []readX509Attributes(Node currentCA)
  throws ParsingException, MalformedURLException {
    HashSet attributes = new HashSet();
    String name = ((Element)currentCA).getAttribute("name");
    
    if (name.equals(null)) {
      throw new ParsingException("Unable to determine the name of the CA");
    }
    
    GenCa ca = new GenCa(new DistinguishedName(name));
    if(currentCA.hasChildNodes()){
      NodeList allAttrsDirs = currentCA.getChildNodes();
      for(int j = 0; j < allAttrsDirs.getLength(); j++){
        Node child = allAttrsDirs.item(j);
        
        if(child.getNodeName().equals("dnattr")){
          String x509Attribute = child.getFirstChild().getNodeValue();
          GenAttribute attr = new GenAttribute("X509", x509Attribute);
          GenX509Value val = new GenX509Value();
          
          val.addGenCa(ca);
          attr.addValue(val);
          attributes.add(attr);
        } else if(child.getNodeName().equals("dir")){
          String dir = child.getFirstChild().getNodeValue();
          
          if (DEBUG){
            System.err.println("saw dir " + dir);
          }
          
          String cDir = cleanStr(dir);
          ca.addDir(new Directory(new URL(cDir).toString()));
        }
      }
    }
    GenAttribute []retAttrs = new GenAttribute[attributes.size()];
    
    attributes.toArray(retAttrs);
    return retAttrs;
  }
  
  /**
   * Remove blank spaces and quotations around the String
   *
   * @param dir String
   * @return String with quotations removed
   */
  private String cleanStr(String stringToClean) {
    String str = stringToClean;
    
    while(str.charAt(0) == ' '){
      str = str.substring(1);
    }
    
    if(str.charAt(0) == '\"') {
      str = str.substring(1);
    }
    
    while(str.charAt(str.length()-1) == ' '){
      str = str.substring(0, str.length()-2);
    }
    
    if(str.charAt(str.length()-1) == '\"') {
      str = str.substring(0,str.length()-1);
    }
    
    return str;
  }
  
  /**
   * Parses all Attributes. It includes a Name tag followed by a
   * Value tag.
   *
   * @return GenAttribute[] the array of GenAttributes read in.
   * @see ResFileParser#readSigner parses the 'signer' element
   */
  private GenAttribute readAttr(Node currentAttr) throws ParsingException {
    NamedNodeMap attrAttrs = currentAttr.getAttributes();
    
    if (attrAttrs == null) {
      throw new ParsingException("Unable to determine the name and value of the attribute");
    }
    
    Node nameNode = attrAttrs.getNamedItem("name");
    String name = nameNode.getNodeValue();
    
    Node valNode = attrAttrs.getNamedItem("value");
    String val = valNode.getNodeValue();
    
    GenAttribute attribute = new GenAttribute("AKENTI", name);
    GenValue value = new GenValue(val);
    attribute.addValue(value);
    
    if(!(currentAttr.hasChildNodes())) {
      throw new ParsingException("resattrs.xml must have at least one Signer");
    }
    
    NodeList allSigners = currentAttr.getChildNodes();
    for(int j = 0; j < allSigners.getLength(); j++) {
      Node signer = allSigners.item(j);
      
      if(signer.getNodeName().equals("signer")){
        AkentiPrincipal akPrin = readSigner(signer);
        value.addAkentiPrincipal(akPrin);
      } else {
        throw new ParsingException("Unable to read Signers");
      }
    }
    
    return attribute;
  }
  
  /**
   * Parse a single signer tag. This includes a NAME tag and a CA tag.
   *
   * @return AkentiPrincipal that was parsed
   */
  private AkentiPrincipal readSigner(Node signerNode) throws ParsingException {
    if(!(signerNode.hasChildNodes())) {
      throw new ParsingException("resattrs.xml must have at least one Signer");
    }
    
    NodeList nameAndCa = signerNode.getChildNodes();
    String dn = new String();
    String caDn = new String();
    boolean dnBool = false;
    boolean caDnBool = false;
    for (int i = 0; i < nameAndCa.getLength(); i++) {
      Node node = nameAndCa.item(i);
      
      if(node.getNodeName().equals("name") && !dnBool){
        dn = node.getFirstChild().getNodeValue();
        dnBool = true;
      } else if(node.getNodeName().equals("ca") && !caDnBool) {
        caDn = node.getFirstChild().getNodeValue();
        caDnBool = true;
      } else {
        throw new ParsingException("Unable to read Signer's Name or CA");
      }
      
      if(dnBool && caDnBool)
        break;
    }
    
    return new AkentiPrincipal(new DistinguishedName(dn), new DistinguishedName(caDn));
  }
  
  /**
   * Reads all GenActions from the resInfo file
   *
   * @return GenActions all actions read from the resInfo file.
   */
  private GenActions readActions() throws ParsingException{
    GenActions retActions = new GenActions();
    
    NodeList allActions = tree.getElementsByTagName("actions");
    if(allActions.getLength() <= 0) {
      throw new ParsingException("resattrs.xml must have at least one Action");
    }
    
    for(int i = 0; i < allActions.getLength(); i++) {
      Node currentAction = allActions.item(i);
      NodeList acts = currentAction.getChildNodes();
      
      if(acts.getLength() <= 0) {
        throw new ParsingException("resattrs.xml file must have at least one Action");
      }
      
      for (int j = 0; j < acts.getLength(); j++) {
        Node a = acts.item(j);
        
        if(a.getNodeName().equals("act")){
          String act = a.getFirstChild().getNodeValue();
          retActions.addAction(act);
        } else {
          throw new ParsingException("Unable to read Actions");
        }
      }
    }
    
    return retActions;
  }
  
  /**
   * main -- for testing purposes
   */
  public static void main(String[] args) {
    try {
      
      ResFileParser p = new ResFileParser("resattr.xml");
      ResFileInfo info = new ResFileInfo();
      
      p.loadResFileInfo(info);
      System.out.println("\n" + info.paramString() + " \n");
    } catch(Exception ex) {
      ex.printStackTrace();
    }
  }
  
}