/**
 *  @author Jordan Thompson (JordanCThompson@yahoo.com)
 */

package lbl.akenti.policyManager.manager;

import java.net.URL;
import lbl.akenti.certs.AttributeCertGenerator;
import lbl.akenti.certs.elements.AkentiIssuer;
import lbl.akenti.certs.elements.AkentiPrincipal;
import lbl.akenti.policyManager.base.GenAttribute;
import lbl.akenti.policyManager.base.GenInfo;
import lbl.akenti.policyManager.base.GenValue;
import lbl.akenti.policyManager.widgets.AbstractController;
import lbl.akenti.policyManager.widgets.LBLIssuerPanel;
import lbl.akenti.policyManager.widgets.LBLResourcePanel;
import lbl.akenti.policyManager.widgets.LBLReviewPanel;
import lbl.akenti.policyManager.widgets.LBLSavePanel;
import lbl.akenti.policyManager.widgets.LBLTimePanel;
import lbl.akenti.policyManager.widgets.LBLTopFrame;
import lbl.akenti.policyManager.widgets.attr.AtConsts;
import lbl.akenti.policyManager.widgets.attr.AtUserPanel;
import lbl.akenti.policyManager.widgets.attr.AtValuePanel;
import lbl.akenti.util.UTCTime;

/**
 */
public class AttrCertController extends AbstractController implements AtConsts {
    
    public AttrCertController() {
        super(7);     // We will be using 7 panels.
    }
    
    private LBLTopFrame frame;
    
    // panels used by AttrCertController
    private LBLResourcePanel lblResourcePanel;
    private AtValuePanel atValuePanel;
    private LBLIssuerPanel lblIssuerPanel;
    private AtUserPanel atUserPanel;
    private LBLTimePanel lblTimePanel;
    private LBLReviewPanel lblReviewPanel;
    private LBLSavePanel lblSavePanel;
    
    public void start() {
        frame = LBLTopFrame.getInstance("Attribute Certificate Generator");
        
        // set controller (allows LBLTopFrame to call methods here)
        frame.setController(this);
        
        // initialize panels
        lblResourcePanel = new LBLResourcePanel(frame, HEADERS[RESOURCE_INDEX]);
        
        atValuePanel = new AtValuePanel(frame, HEADERS[ATTRIBUTE_INDEX]);
        
        lblIssuerPanel = new LBLIssuerPanel(frame, HEADERS[ISSUER_INDEX]);
        
        atUserPanel = new AtUserPanel(frame, HEADERS[USER_INDEX]);
        
        lblTimePanel = new LBLTimePanel(frame, HEADERS[TIME_INDEX]);
        
        lblReviewPanel = new LBLReviewPanel(frame, HEADERS[REVIEW_INDEX]);
        
        lblSavePanel = new LBLSavePanel(frame, HEADERS[SIGN_INDEX]);
        
        // add panels in this specific order
        frame.addPanel(lblResourcePanel, "LBLResourcePanel");
        frame.addPanel(atValuePanel, "AtValuePanel");
        frame.addPanel(lblIssuerPanel, "LBLIssuerPanel");
        frame.addPanel(atUserPanel, "AtUserPanel");
        frame.addPanel(lblTimePanel, "LBLTimePanel");
        frame.addPanel(lblReviewPanel, "LBLReviewPanel");
        frame.addPanel(lblSavePanel, "LBLSavePanel");
    }
    
    public void clearLBLResourcePane() {
        lblResourcePanel.clear();
    }
    public LBLTopFrame getFrame() {
        return frame;
    }
    public char[] getPassword() {
        return lblSavePanel.getPassword();
    }
    public String getP12Filename() {
        lblSavePanel.save();
        return lblSavePanel.getKeyFileName();
    }
    public String getSaveLocation() {
        return lblSavePanel.getCertFileName();
    }
    protected boolean loadInputsIntoPanel(int index) throws Exception {
        switch(index) {
            case ATTRIBUTE_INDEX:
                URL base = new URL(lblResourcePanel.getBaseURL());
                String resource = lblResourcePanel.getResource();
                
                if (info == null || !(info.hasResource(resource) && info.hasBaseURL(base))) {
                    // Make sure we load the GenInfo first.
                    GenInfo tempInfo = new GenInfo(resource, base);
                    
                    tempInfo.loadResFileInfo();
                    tempInfo.loadPolicies();
                    
                    this.info = tempInfo;
                    
                    GenAttribute[] attributes = info.getGenAttributes();
                    
                    atValuePanel.clear();
                    
                    for (int i = 0; i< attributes.length; i++) {
                        if (!attributes[i].isX509Attribute()) {
                            atValuePanel.addAttribute(attributes[i]);
                        }
                    }
                    
                    frame.clearBits();
                }
                lblResourcePanel.save();
                return false;
                
            case ISSUER_INDEX:
                GenValue value = atValuePanel.getGenValue();
                lblIssuerPanel.setOptional(true);
                lblIssuerPanel.setIssuers(value.getAkentiPrincipals());
                return false;
                
            case USER_INDEX:
                return true;
                
            case TIME_INDEX:
                return true;
                
            case REVIEW_INDEX:
                AttributeCertGenerator gen = new AttributeCertGenerator();
                gen.setVersionNumber(2);
                gen.setUniqueID("UID");
                
                UTCTime utc = new UTCTime();
                
                gen.setNotBefore(utc);
                gen.setNotAfter(new UTCTime(utc, lblTimePanel.getLifeTime()));
                
                
                AkentiPrincipal issuer = lblIssuerPanel.getIssuer();
                
                if (issuer != null) {
                    gen.setIssuer(new AkentiIssuer(issuer));
                }
                
                
                gen.setAttribute(atValuePanel.getAttributeName());
                gen.setValue(atValuePanel.getValueName());
                
                gen.setPrincipal(atUserPanel.getAkentiPrincipal());
                
                lblReviewPanel.setCertificate(gen.generate());
                this.gen = gen;
                return false;
                
            case SIGN_INDEX:
                lblSavePanel.clear();
                //lblSavePanel.doEnable();
                return false;
                
            default:
                throw new RuntimeException();
        }
    }
    
    /**
     */
    protected String getHelpMessage(int index) {
        return helpMess[index];
    }
    
    public void showDialog(String title, String info) {
        frame.showDialog(title, info);
    }
    
    public LBLSavePanel getSavePanel() {
        return lblSavePanel;
    }
    public void cancel() {
        frame.cancel();
    }
    public void checkEnableButton(int index) {
        switch(index) {
            case ATTRIBUTE_INDEX:
                atValuePanel.doEnable();
                break;
                
            case ISSUER_INDEX:
                break;
                
            case USER_INDEX:
                atUserPanel.doEnable();
                break;
                
            case TIME_INDEX:
                break;
                
            case REVIEW_INDEX:
                break;
                
            case SIGN_INDEX:
                lblSavePanel.doEnable();
                break;
                
            default:
                throw new RuntimeException("Error in AttrCertController.checkEnable");
        }
    }
}
