package lbl.akenti.protocolHandler;

import java.io.IOException;
import lbl.akenti.messaging.AkentiRequest;
import lbl.akenti.messaging.AkentiResponse;
import lbl.akenti.messaging.CheckAccessRequest;
import lbl.akenti.messaging.CheckAccessResponse;
import lbl.akenti.messaging.CheckPolicyRequest;
import lbl.akenti.messaging.CheckPolicyResponse;
import lbl.akenti.messaging.CreateResourceRequest;
import lbl.akenti.messaging.CreateResourceResponse;
import lbl.akenti.messaging.DeletePolicyCertificateRequest;
import lbl.akenti.messaging.DeletePolicyCertificateResponse;
import lbl.akenti.messaging.DeleteResourceRequest;
import lbl.akenti.messaging.DeleteResourceResponse;
import lbl.akenti.messaging.GetResourceInfoRequest;
import lbl.akenti.messaging.GetResourceInfoResponse;
import lbl.akenti.messaging.ResourceBrowseRequest;
import lbl.akenti.messaging.ResourceBrowseResponse;
import lbl.akenti.messaging.UploadPolicyCertificateRequest;
import lbl.akenti.messaging.UploadPolicyCertificateResponse;

/**
 * An interface for objects that processes AkentiRequests and
 * spit out the appropriate AkentiResponse.
 */
abstract public class ProtocolHandler {
  
  /**
   * Checks a users access to a resource.
   * @param req the request
   * @throws IOException not thrown
   * @return the response
   */
  abstract public CheckAccessResponse
  process(CheckAccessRequest req) throws IOException;
  
  /**
   * Checks a resource and returns all associated certificates.
   * @param req the request
   * @throws IOException not thrown
   * @return the response
   */
  abstract public CheckPolicyResponse
  process(CheckPolicyRequest req) throws IOException;
  
  /**
   * Returns the resources immediatly below the one mentioned in
   * the request.
   * @param req the request
   * @throws IOException not thrown
   * @return the response
   */
  abstract public ResourceBrowseResponse
  process(ResourceBrowseRequest req) throws IOException;
  
  /**
   * Creates the resource mentioned in the request.
   * @param req the request
   * @throws IOException not thrown
   * @return the response
   */
  abstract public CreateResourceResponse
  process(CreateResourceRequest req) throws IOException;
  
  /**
   * Deletes the resource mentioned in the request.
   * @param req the request
   * @throws IOException not thrown
   * @return the response
   */
  abstract public DeleteResourceResponse
  process(DeleteResourceRequest req) throws IOException;
  
  /**
   * Uploads the policy certificate in the request.
   * @param req the request
   * @throws IOException not thrown
   * @return the response
   */
  abstract public UploadPolicyCertificateResponse
  process(UploadPolicyCertificateRequest req) throws IOException;
  
  /**
   * Searches for a policy certificate in the resource mentioned
   * in the request. If a policy certificiate is found it is
   * deleted. Note: this will not delete a policy certificate
   * if the resource inheirits it's policy from a higher resource.
   * @param req the request
   * @throws IOException not thrown
   * @return the response
   */
  abstract public DeletePolicyCertificateResponse
  process(DeletePolicyCertificateRequest req) throws IOException;
  
  /**
   * Searches for the hint file for the resource named in the
   * request and returns it if it exists.
   * @param req the request
   * @throws IOException not thrown
   * @return the response
   */
  abstract public GetResourceInfoResponse
  process(GetResourceInfoRequest req) throws IOException;
  
  
  /**
   * Processes a single request.
   * @param req the request
   * @throws IOException never thrown
   * @return the response
   */
  abstract public AkentiResponse
  process(AkentiRequest req) throws IOException;
  
  /**
   * Processes multiple requests.
   * @param requests the requests
   * @throws IOException never thrown
   * @return the responses
   */
  abstract  public AkentiResponse[]
  process(AkentiRequest[] requests) throws IOException;
  
  private static int id = 0;
  
  /**
   * Returns a unique id. Used to generate
   * <CODE>id</CODE>s for AkentiRequests.
   * @return a unique id
   */  
  synchronized public static int getID() {
    return id++;
  }
  
}