package lbl.akenti.servers.AKMonitor.gui;

/**
  * ResourceViewer.java.
  *
  * by Abdelilah Essiari
  *
  **/
import javax.swing.JPanel;
import javax.swing.JLabel;
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.Color;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Vector;
import java.util.NoSuchElementException;

public class ResourceViewer extends JPanel {
  protected static final String
    phases []  =
      {
         "Collectiong policy elements: ",
         "Verifying policy elements: ",
         "Verifying access: ",
         "Capabilities: "
      };

  protected static int defaultNumOfStatusPanels = 4;
  protected static String defaultHeader = "Resource:";
  protected static Vector cache = new Vector();

  /*  
   * each <code>StatusPanel</code> panel is associated 
   * with a phase in access control and  will basically 
   * contain <code>StatusCanvas</code> widgets.
   */  
  protected int numOfStatusPanels;
  
  /*
   * header to be to be concatenated to name of
   * the resource,
   */
  protected String header;

  /*
   * each one of these is associated with an access
   * control phase.
   */
  protected StatusPanel [] panels;
  
  /*
   * label to view the resource name.
   */
  protected JLabel label;
  
  /*
   * mouse listener to be added to the canvases.
   */
   protected MouseListener mouseListener;

   protected TextArea area;
  
  /*
   * Constructor.
   */
  public ResourceViewer() {
    numOfStatusPanels = defaultNumOfStatusPanels;
    header = defaultHeader;
    resourceViewerInit();
  }

  /*
   * intialize the resource viewer.
   * 
   * 1) add label used to view the name of the resource.
   * 2) add StatusPanels to view the logs associated with
   *    the resource.
   **/
  protected void resourceViewerInit() {
    setLayout(new BorderLayout(5, 5));
    
    label = new JLabel();
    label.setForeground(Color.black);
    label.setVerticalTextPosition(label.TOP);
    label.setHorizontalAlignment(label.LEFT);
    label.setHorizontalTextPosition(label.LEFT);
    add("North", label);

    JPanel tmpPanel;

    tmpPanel = new JPanel();
    tmpPanel.setLayout(new GridLayout(numOfStatusPanels, 1, 10, 10));

    panels = new StatusPanel[numOfStatusPanels];

    for (int i = 0; i < numOfStatusPanels; i++) {
      tmpPanel.add(panels[i] = new StatusPanel());
      setStatusPanelTitle(i, phases[i]);
    }

    area = new TextArea("Log");
    mouseListener = new MouseListener(area);
    add("Center", tmpPanel);
  }

  /*
   * set the name of resource.
   *
   * @param name of resource.
   **/
  public void setResourceName(String name) {
    StringBuffer sb = new StringBuffer(header);
    
    sb.append(name);
    label.setText(sb.toString());
  }

  /*
   * set the name of each <code>StatusPanel</code> panel.
   */
  public void setStatusPanelTitle(int index, String name) {
    panels[index].setTitle(name);
  }

  /*
   * remove all <code>StatusCanvas</code> widgets from the 
   * <code>StatusPanel</code> panels and put them in the cache.
   */
  public void clear() {
    StatusCanvas []canvases;

    label.setText("");
    mouseListener.deselectCanvas();

    for (int i = 0; i < numOfStatusPanels; i++) {
      canvases = panels[i].canvases();
      panels[i].clear();
      
      for (int j = 0; j < canvases.length; j++) {
        canvases[j].setStatusEvent(null);
        cache.addElement(canvases[j]);
      } 
    }
  }

  public void dispose() {
    area.dispose();
  }

  /*
   * add an <code>StatusEvent</code>.
   * 
   * @param event StatusEvent
   * @return the <code>StatusCanvas> that hosts the event.
   **/
  public StatusCanvas addStatusEvent(StatusEvent event) {
    int level = event.getLevel();

    if (level < 0 || level >= numOfStatusPanels) {
      throw new IllegalArgumentException("bad event:level = " + level);
    }

    StatusCanvas canvas;
    
    /* check cache first */
    if (cache.isEmpty()) {
      canvas = panels[level].addStatusCanvas(event);
    } else {
      canvas = (StatusCanvas) cache.firstElement();
      cache.removeElementAt(0);
      canvas.setStatusEvent(event);
      panels[level].addStatusCanvas(canvas);
    }
    
    canvas.addMouseListener(mouseListener);
    return canvas;
  }
  
  public void next() {
    StatusCanvas canvas = mouseListener.getSelectedCanvas();
     
    if (canvas == null) {
      return;
    }

    canvas = getNextStatusCanvas(canvas);
    
    if (canvas != null) {
      mouseListener.statusCanvasSelected(canvas); 
    }
  }
  
  public void previous() {
    StatusCanvas canvas = mouseListener.getSelectedCanvas();

    if (canvas == null) {
      return;
    }

    canvas = getPreviousStatusCanvas(canvas);
    
    if (canvas != null) {
      mouseListener.statusCanvasSelected(canvas); 
    }
  }

  /*
   * get <code>StatusCanvas</code> that was added before 
   * the canvas specified by the parameter.
   *
   * @return previous canvas if found.
   *         null otherwise.
   **/
  protected StatusCanvas getPreviousStatusCanvas(StatusCanvas canvas) {
    int level = canvas.getStatusEvent().getLevel();
    
    StatusCanvas [] canvases = panels[level].canvases();
    
    if (canvas == canvases[0]) {
      while (level-- != 0) {
        try {
          return panels[level].lastCanvas();
        } catch(NoSuchElementException e) { 
        }
      }

      return null;
    }

    StatusCanvas prev = null;

    for (int i = 1; i < canvases.length ; i++) { 
      if (canvas == canvases[i]) {
        prev = canvases[i - 1];
      } 
    }
   
    return prev;
  }
 
  /*
   * get <code>StatusCanvas</code> that was added after 
   * canvas.
   *
   * @return next canvas if found.
   *         null otherwise.
   **/
  protected StatusCanvas getNextStatusCanvas(StatusCanvas canvas) {
    int level = canvas.getStatusEvent().getLevel();

    StatusCanvas [] canvases = panels[level].canvases();
    
    if (canvas == canvases[canvases.length - 1]) {
      while (level++ != numOfStatusPanels - 1) {
        try { 
          return panels[level].firstCanvas();
        } catch(NoSuchElementException e) {
        }
      }

      return null;
    }

    StatusCanvas next = null;

    for (int i = 0; i < canvases.length - 1; i++) { 
      if (canvas == canvases[i]) {
        next = canvases[i + 1];
      } 
    }
    
    return next;
  }
  

  /**
    * mouse listener to keep track of the selected canvas;
    * when the user clicks on a little canvas, we change the 
    * its background color to black and we view its contents
    * in a TextArea.
    */
  protected static class MouseListener extends MouseAdapter {
    StatusCanvas selected; 
    TextArea area;
    
    public MouseListener(TextArea area) {
      this.area = area;
    }

    /* this is the only event that we handle */
    public void mouseClicked(MouseEvent e) {
      statusCanvasSelected((StatusCanvas) e.getSource());
    }
    
    /* returns the selected canvas */
    public StatusCanvas getSelectedCanvas() {
      return selected;
    }
    
    /* deselect canvas */
    public void deselectCanvas() {
      if (selected != null) {
        selected.setBackgroundColor();
        selected.invalidate();
        selected.repaint();
        selected = null;
        area.setVisible(false);
      }
    }

    /*
     * change the color of the selected canvas, both the 
     * old and the new.
     * view contents in text area.
     **/ 
    public void statusCanvasSelected(StatusCanvas canvas) {
      if (selected != canvas) {
        deselectCanvas();
        selected = canvas;
        selected.setBackground(Color.black);
        selected.invalidate();
        selected.repaint();
        area.setText(selected.getStatusEvent().toString());
        area.setVisible(true);
      }
    }
  }
}

