package lbl.akenti.servers.AKMonitor.server;

import lbl.akenti.servers.AKMonitor.util.AkentiEventConstants;
import lbl.akenti.servers.AKMonitor.util.AppletConnection;
import lbl.akenti.servers.AKMonitor.util.AkentiEventParser;
import lbl.akenti.servers.AKMonitor.util.AKInfo;
import lbl.akenti.servers.AKMonitor.util.AkentiEvent;
import lbl.akenti.servers.AKMonitor.util.KeyFile;

import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;

import java.io.BufferedOutputStream;
import java.io.OutputStream;
import java.io.InputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import java.net.Socket;

/**
  * Handles the connection between the Akenti engine (that provide log messages)
  * and Applets tha are waiting for log messages
  * @author Guillaume Farret
  * @version August 2001
  */
public class ConnectionHandler extends Thread implements AkentiEventConstants {
  
  private KeyFile keyFile = null; // keyfile used to comment the event
  private AkentiEventParser aep = null; // contain the parser for the currnt session
  private int id; //contain the unique id of the session currently handled

  private static Hashtable appletConnections = new Hashtable(); // contain Sockets of waiting applet
  
  private static int uniqueId = 0;// unique id counter

  private static synchronized int getUniqueId() { 
    uniqueId++;

    if (uniqueId == Integer.MAX_VALUE) {
      uniqueId = 1;
    }

    return uniqueId;
  }

  
/**
  * Constructor
  * @param inputStream input stream, usually come from the LogServer Socket
  */  
  public ConnectionHandler(InputStream inputStream)
  throws IllegalArgumentException, Exception {
    if (inputStream == null) {
      throw new IllegalArgumentException();
    }

    keyFile = KeyFile.getKeyFile();

    if (keyFile == null) {
      throw new RuntimeException("keyFile not initialized");
    }

    id = getUniqueId();
    aep = new AkentiEventParser(inputStream);
  }

/**
  * Main thread that handle a connection between the incoming logs 
  * and all waiting applet
  */
  public void run() {
    try {
      StringBuffer sb = new StringBuffer();
      
      //we get the first event to fill the appletSockets Vector
      AkentiEvent event = aep.nextEvent(sb);

      if (event==null) {
        System.err.println("WE HAVE A PROBLEM");
        return;
      }

      String uid = (String) event.table.get("UID");
      String resource = (String) event.table.get("RESOURCE");
       
      System.err.println(uid + ':' + resource + ": LOGs incoming, connection id: " + id);
          
      Vector currentAppletConnections = (Vector) getAppletConnections(uid);

      while (event != null) {
        //filling the AkentiEvent with info from the KeyFile  
        AKInfo aki = keyFile.getAKInfo(event.key);
        
        if (aki != null) {
          event.setDescription(aki.getDescription());
          event.setFail(aki.getFail());
          event.setSuccess(aki.getSuccess());
        }
        
        event.setId(id);
       
        //write the event to all outputStreams
        for (int i=0; i < currentAppletConnections.size(); i++) {
          AppletConnection ac = (AppletConnection) currentAppletConnections.elementAt(i);
          ac.P();
          try {
            DataOutputStream out = ac.getDataOutputStream();
            //event.writeToStream(new DataOutputStream(System.err));
            event.writeToStream(out);
            //out.flush();
            ac.V();

          } catch (IOException e) { //if an applet logout, then remove it from the list.
            removeAppletConnection(uid, ac);
            currentAppletConnections.remove(i--);
            ac.getSocket().close();

          } 
        }

        event = aep.nextEvent(sb);
      }
      
      //warning applet of the end of a connection
      for (int i=0; i< currentAppletConnections.size(); i++) {
        AppletConnection ac = (AppletConnection) currentAppletConnections.elementAt(i);
        ac.P();
        try {
          DataOutputStream out = ac.getDataOutputStream(); 
          String msgEOF = -id + "##";
          synchronized (out) {
            out.writeInt(msgEOF.length());
            out.writeBytes(msgEOF);
            out.flush();
          }
          ac.V();
        } catch (IOException e) {
          //applet already deconnected
          removeAppletConnection(uid,ac);
          currentAppletConnections.remove(i--);
          ac.getSocket().close();
        } 
      }//endFor
      
      //write the logs (could be a file)
      System.out.println("******\n");
      System.out.println(sb.toString());
      System.out.flush();
      System.out.close();
      System.err.println(uid + " : processing LOGs finished, connection id: " + id);

    }catch (Exception e) {
      System.err.println(" ERROR ");
      e.printStackTrace(System.err);
    }
  }

/**
  * method to store a Socket in the correct Vector according 
  * to the UID that it provide.
  *
  * @param ac AppletConnection to be stored in hashtable
  * @param key hashtable key
  * @return boolean true if the Connection has been stored,
  *                 false if the maximum storage amount has been reached
  */

  public static boolean storeAppletConnection(String key, AppletConnection ac) {
    Vector temp;

    synchronized (appletConnections) {
      if (appletConnections.containsKey(key)) {
        temp = (Vector) appletConnections.get(key);
      } else {
        temp = new Vector();
        appletConnections.put(key,temp);
      }

      // Limit the number of applets per uid
      if (temp.size() >= 5) {
        return false;
      }

      temp.addElement(ac);
    }

    return true;
  }

/**
  * method to remove a unique socket so from the Vector  of 
  * Socket thats belong to applet waiting for the log: key
  *
  * @param key specifying the AppletConnection Vector
  * @param so specifying the Socket that we want to remove
  */
  public static boolean removeAppletConnection(String key, AppletConnection ac) {
    Vector temp = (Vector) appletConnections.get(key);

    if (temp == null) {
      return false;
    }

    boolean ret = temp.remove(ac);

    System.err.println("removed connection: " + key + " : " + ac);
      
    if (temp.size() == 0) {
      appletConnections.remove(key);
    }

    return ret;
  }
  
/**
  * method to retrieve a Vector of Socket thats belong to
  * applet waiting for the log: key
  *
  * @param key specifying the connection that we expect to get
  * @return Vector of Socket waiting for the log : key
  */    
  public static Vector getAppletConnections(String key) {
    Vector ret = (Vector) appletConnections.get(key);
    if (ret != null) {
      return (Vector) ret.clone();
    } else {
      return new Vector();
    }
  }

  public static synchronized Enumeration getKeys() {
     return appletConnections.keys();
  }

/**
  * thread that go through the static Hashtable to remove too old connection
  */
  public static void startGarbageCollector() {
    new Thread() {
      public void run() {
        System.err.println("Starting key garbaging agent ...");
        while(true) {
          try { 
            Thread.currentThread().sleep(5*1000);//sleep during 5 seconds 
          } catch(InterruptedException e) { 
            System.err.println(e);
          }
          Enumeration keys = getKeys();

          while (keys.hasMoreElements() == true) {
            String key = (String)keys.nextElement(); 
            Vector tmp = (Vector)appletConnections.get(key);
            
            for (int i=0; i<tmp.size(); i++) {
              AppletConnection ac = (AppletConnection)tmp.elementAt(i);

              //if connection inactive for more than 10 seconds then remove it
              synchronized (ac) {
                if (ac.getUsed() == 0) {//if AppletConnection not used
                  if ((System.currentTimeMillis()-ac.getLastTimeUsed()) > 10*1000) {//if too old
                    System.err.println("Garbage ... removing too old connection");
                    ConnectionHandler.removeAppletConnection(key,ac);
                  } 
                }
              }
            }
          }
        }
      }
    }.start();
  }
}
