package lbl.akenti.servers.ResDefServer;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Enumeration;
import java.util.StringTokenizer;
import lbl.akenti.util.AkentiConstants;

/**
 * This simple http handler handles GET requests only.
 *
 * @author Jiewei Lin
 */
public class HttpHandler implements AkentiConstants {
  /**
   * Default constructor.
   */
  public HttpHandler() {
  }
  
  /**
   * Handles GET requests.
   *
   * @param request contains HTTP request info
   * @param response uses to write to the client
   * @exception MovedPermanentlyException is thrown if requesting for a directory but
   *   missing "/" at the end of the URI.
   * @exception IOException is thrown is there's a problem when reading from InputStream
   *   or when writing to OutputStream.
   * @exception ForbiddenException is thrown if file or directory is not found.
   * @exception ConfigurationException is thrown if either PolicyFileName or
   *   ResAttrFileName retrieved from System.Property is empty.
   */
  public void doGet(HttpRequest request, HttpResponse response)
  throws MovedPermanentlyException, IOException, ForbiddenException,
  ConfigurationException {
    if (request == null || response == null) {
      throw new IllegalArgumentException();
    }
    
    String uri = request.getURI();
    
    String policyFileName = System.getProperty("PolicyFileName", PolicyFileName);
    String resAttrFileName = System.getProperty("ResAttrName", ResAttrFileName);
    
    if (policyFileName.equals("")) {
      throw new ConfigurationException(
      "Must provide a <i>PolicyFileName</i> property in config file"
      );
    }
    
    if (resAttrFileName.equals("")) {
      throw new ConfigurationException(
      "Must provide a <i>ResAttrName</i> property in config file"
      );
      
    }
    
    final String CRLF = "\r\n";
    StringBuffer entityBody = new StringBuffer
    ("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2 Final//EN\">" + CRLF);
    
    entityBody.append("<HTML>" + CRLF + "<HEAD><TITLE>Index of " +  uri +
    "</TITLE></HEAD>" +  CRLF + "<BODY>" +  CRLF +
    "<H1>Index of " +  uri + "</H1>" +  CRLF +
    "<PRE>" +  CRLF + "<HR>" + CRLF);
    
    RootResourceParser parser = RootResourceParser.getInstance();
    
    if (uri.equals("/")) {
      for (Enumeration e = parser.getRootResources(); e.hasMoreElements();) {
        String root = (String)e.nextElement();
        entityBody.append("[DIR] <A HREF=\"" + root + "/\">" + root + "</A>" + CRLF);
      }
      
      entityBody.append("</PRE>" + CRLF + "<HR>" + CRLF + "</BODY>" + CRLF + "</HTML>");
      response.getOutputStream().write(entityBody.toString().getBytes());
      response.setContentType("text/html");
      response.setStatusCode(200);
      return;
    }
    
    StringTokenizer st = new StringTokenizer(uri, "/");
    
    String name = st.nextToken();
    
    if (parser.hasRootResource(name)) {
      name = parser.getRootFullPath(name) + uri;
    } else {
      throw new ForbiddenException();
    }
    
    File file = new File(name);
    
    if (!file.exists()) {
      throw new ForbiddenException();
    } else if (file.isFile()) {
      if (uri.endsWith(policyFileName) || uri.endsWith(resAttrFileName)) {
        response.setContentType(getContentType(uri));
        
        FileInputStream fis = new FileInputStream(file);
        
        sendBytes(fis, response.getOutputStream());
        fis.close();
        response.setStatusCode(200);
        return;
      } else {
        throw new ForbiddenException();
      }
    }
    
    if (!name.endsWith("/")) {
      throw new MovedPermanentlyException();
    }
    
    if (!uri.equals("/")) {
      String parent = new File(uri).getParent();
      
      if (!parent.endsWith("/")) {
        parent += "/";
      }
      
      entityBody.append("[DIR] <A HREF=\"" + parent + "\">Parent Directory</A>" + CRLF);
    }
    
    String[] list = file.list();
    
    for (int i = 0; i < list.length; i++) {
      File temp = new File(name + list[i]);
      if (temp.isDirectory()) {
        entityBody.append("[DIR] <A HREF=\"" + list[i] + "/\">" + list[i] + "</A>" + CRLF);
      } else {
        if (list[i].equalsIgnoreCase(policyFileName)) {
          entityBody.append("[   ] <A HREF=\"" + policyFileName + "\">" + policyFileName +
          "</A>" + CRLF);
        }
        
        if (list[i].equalsIgnoreCase(resAttrFileName)) {
          entityBody.append("[   ] <A HREF=\"" + resAttrFileName + "\">" + resAttrFileName +
          "</A>" + CRLF);
        }
      }
    }
    
    entityBody.append("</PRE>" + CRLF + "<HR>" + CRLF + "</BODY>" + CRLF + "</HTML>");
    response.getOutputStream().write(entityBody.toString().getBytes());
    response.setContentType("text/html");
    response.setStatusCode(200);
  }
  
  /**
   * Determines file type.
   *
   * @param fileName name of the file
   * @return String file type
   */
  private String getContentType(String fileName) {
    if (fileName.endsWith(".gif")) {
      return "image/gif";
    } else if (fileName.endsWith(".jpg")) {
      return "image/jpeg";
    } else {
      return "text/html";
    }
  }
  
  /**
   * Sends bytes from FileInputStream fis to OutputStream os.
   *
   * @param fis FileInputStream reads from file.
   * @param os OutputStream sends to the client.
   * @exception IOException is thrown if errors occurred when reading from FileInputStream
   *   fis or writing to OutputStream os.
   */
  private void sendBytes(FileInputStream fis, OutputStream os) throws IOException {
    byte[] buffer = new byte[1024] ;
    int bytes = 0 ;
    
    while ((bytes = fis.read(buffer)) != -1 ) {
      os.write(buffer, 0, bytes);
    }
  }
}