package lbl.akenti.tools;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.security.cert.X509Certificate;
import java.util.Set;
import java.util.TreeSet;
import lbl.akenti.certs.AkentiCertificateFactory;
import lbl.akenti.certs.AttributeCertificate;
import lbl.akenti.certs.CapabilityCertificate;
import lbl.akenti.certs.UseCondCertificate;
import lbl.akenti.certs.elements.AkentiPrincipal;
import lbl.akenti.certs.elements.ConditionalActions;
import lbl.akenti.policyEngine.PolicyContext;
import lbl.akenti.policyEngine.ResourcePolicy;
import lbl.akenti.policyEngine.impl.AkentiEngine;
import lbl.akenti.util.ArgumentParser;
import lbl.akenti.util.LDAPSearchUtil;
import lbl.akenti.util.X509CertReader;

/**
 * example: CheckAccess -f conf.txt -r "R1" -x cert -R read -R write
 * options:
 * -f arg           - akenti engine config file
 * -r arg           - resource
 * [-x arg]         - user's x509 cert chain
 * [-u arg -c arg]  - user's DN and user's CADN
 * [-R arg]*        - a specific action(s)
 * [-A arg -V arg]* - attribute/value pair
 * [-o arg]         - certs' output file (defaults to stdout)
 * [-P]             - show all certs used in making the decision
 * [-q]             - show concise output
 * [-v]             - verbose
 * [-h]             - prints this message
 *
 * *multiple instances of this flag allowed
 *
 * @author Tommaso Sciortino
 */
class CheckAccess {
  
  /**
   * Use this program to check a user's access to a resource.
   * @param args command line arguments
   */  
  public static void main(String[] args) {
    String confFileName = null;
    String resource = null;
    String x509CertFileName = null;
    String dn = null;
    String cadn = null;
    Set desiredActions = new TreeSet();
    String[] attribute = null;
    String[] value = null;
    String output = null;
    
    boolean detailed = false;
    boolean concise = false;
    boolean verbose = false;
    
    try{
      ArgumentParser parser = new ArgumentParser(args);
      
      if (args.length == 0) {
        printUsage();
        System.exit(2);
      }
      
      char[] flags = new char[] {'f', 'r', 'x', 'u', 'c', 'R', 'A', 'V', 'o', 'P', 'q', 'v', 'h'};
      parser.parseArgumentTokens(flags);
      
      confFileName = parser.getOptionArg('f');
      resource = parser.getOptionArg('r');
      x509CertFileName = parser.getOptionArg('x');
      dn = parser.getOptionArg('u');
      cadn = parser.getOptionArg('c');
      
      String[] temp = parser.getOptionArgs('R');
      for(int i=0; i < temp.length; i++) {
        desiredActions.add(temp[i]);
      }
      
      attribute = parser.getOptionArgs('A');
      value = parser.getOptionArgs('V');
      
      output = parser.getOptionArg('o');
      
      detailed = parser.getOption('P');
      concise = parser.getOption('q');
      verbose = parser.getOption('v');
      
      if (parser.getOption('h')) {
        printUsage();
        System.exit(2);
      } else if (confFileName == null || confFileName.equals("")) {
        System.err.println("Must provide a config file");
        System.exit(2);
      } if ( (x509CertFileName == null || x509CertFileName.equals("") ) &&
      (dn == null || dn.equals("") || cadn == null || cadn.equals("")) ) {
        System.err.println("Must proide a user's x509 certificate or DN/CADN pair");
        System.exit(2);
      } if (parser.getOption('x') && (parser.getOption('u') || parser.getOption('c')) ) {
        System.err.println("Must proide either a user's x509 certificate or DN/CADN pair but not both");
        System.exit(2);
      } if (attribute.length != value.length) {
        System.err.println("The number of attributes must match the number of values.");
        System.exit(2);
      } else if (resource == null || resource.equals("")) {
        System.err.println("Must provide a resource");
        System.exit(2);
      }
      
    } catch (IOException ex) {
      System.err.println(ex.getMessage());
      printUsage();
      System.exit(2);
    }
    
    try {
      
      LDAPSearchUtil.init();
      
      if (verbose) {
        System.err.println("Initializaing AkentEngine from " + confFileName);
      }
      
      StringBuffer sb = new StringBuffer();
      boolean ret = AkentiEngine.init(confFileName, sb);
      
      if (verbose) {
        System.err.println(sb);
      }
      
      if (!ret)  {
        System.exit(1);
      }
      
      AkentiEngine eng = new AkentiEngine();
      eng.setDesiredActions(desiredActions);
      // TODO: set restricted ttributes
      
      AkentiPrincipal user = null;
      
      if (x509CertFileName != null) {
        if (verbose) {
          System.err.println("Reading user's cert chain from " + x509CertFileName);
        }
        
        X509CertReader x509Reader = new X509CertReader();
        X509Certificate x509Cert = x509Reader.readX509Cert(x509CertFileName);
        
        user = new AkentiPrincipal(x509Cert.getSubjectDN(), x509Cert.getIssuerDN());
      } else {
        user = new AkentiPrincipal(dn,cadn);
      }
      
      if (verbose) {
        System.err.println("User DN is " + user.getName().to500String());
        System.err.println("CaDN is " + user.getGuarantor().to500String());
        System.err.println("Calling eng.process for resource " + resource);
      }
      
      ret = eng.checkAccess(resource, user);
      
      System.err.println(eng.getAkentiMessage());
      
      OutputStream os = null;
      
      if (output != null && !output.equals("")) {
        os = new FileOutputStream(new File(output));
      } else {
        os = System.out;
      }
      
      CapabilityCertificate capabilityCert = eng.getCapabilityCertificate();
      
      if (concise) {
        if (ret) {
          printActions(capabilityCert.getUnConditionalActions());
          printActions(capabilityCert.getConditionalActions());
        }
      } else {
        
        if (detailed) {
          
          PolicyContext ctx = eng.getPolicyContext();
          
          ResourcePolicy[] rp = ctx.toResourcePolicies();
          for (int i=0; i < rp.length; i++) {
            AkentiCertificateFactory.writeCertToStream("XML", rp[i].getPolicyCertificate(), os);
            
            UseCondCertificate[] ucs = rp[i].toUseCondCertificates();
            for (int j=0; j < ucs.length; j++) {
              AkentiCertificateFactory.writeCertToStream("XML", ucs[j], os);
            }
          }
          
          AttributeCertificate[] attrs = ctx.toAttributeCertificates();
          for (int i=0; i < attrs.length; i++) {
            AkentiCertificateFactory.writeCertToStream("XML", attrs[i], os);
          }
        }
        
        if (ret) {
          AkentiCertificateFactory.writeCertToStream("XML", capabilityCert, os);
        }
      }
      
      if (ret) {
        System.exit(0);
      } else {
        System.exit(1);
      }
    } catch(RuntimeException ex) {
      System.err.println("SHOULD NEVER HAPPEN");
      ex.printStackTrace();
      System.exit(3);
    } catch(Exception ex) {
      ex.printStackTrace();
      System.exit(3);
    }
  }
  
  static private void printUsage() {
    System.err.println("example: CheckAccess -f conf.txt -r 'R1' -x cert -R read -R write");
    System.err.println("options:");
    System.err.println("  -f arg           - akenti engine config file");
    System.err.println("  -r arg           - resource");
    System.err.println("  [-x arg]         - user's x509 cert chain");
    System.err.println("  [-u arg -c arg]  - user's DN and user's CADN");
    System.err.println("  [-R arg]*        - a specific action(s) ");
    System.err.println("  [-A arg -V arg]* - attribute/value pair");
    System.err.println("  [-o arg]         - certs' output file (defaults to stdout)");
    System.err.println("  [-P]             - show all certs used in making the decision");
    System.err.println("  [-q]             - show concise output");
    System.err.println("  [-v]             - verbose");
    System.err.println("  [-h]             - prints this message");
    System.err.println("*multiple instances of this flag allowed");
  }
  
  static private void printActions(String[] input) {
    System.out.print("Allowed actions: ");
    
    if (input.length == 0) {
      System.out.println("NONE");
      return;
    }
    
    System.out.print(input[0]);
    
    for (int i=1; i < input.length; i++) {
      System.out.print(", " + input[i]);
    }
    
    System.out.println("");
  }
  
  static private void printActions(ConditionalActions[] input) {
    System.out.print("Conditional actions are: ");
    
    if (input.length == 0) {
      System.out.println("NONE");
      return;
    }
    
    System.out.println("");
        
    for (int i=0; i < input.length; i++) {
      System.out.println("    " + input[i].toLogString());
    }
    
    System.out.println("");
  }
}