/*
 * PrintAkentiCertTemplate.java
 *
 * Created on June 24, 2004, 1:08 PM
 */

package lbl.akenti.tools;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import lbl.akenti.certs.AkentiCertificate;
import lbl.akenti.certs.AkentiCertificateFactory;
import lbl.akenti.certs.AttributeCertGenerator;
import lbl.akenti.certs.PolicyCertGenerator;
import lbl.akenti.certs.UseCondCertGenerator;
import lbl.akenti.certs.elements.AkentiIssuer;
import lbl.akenti.certs.elements.AkentiPrincipal;
import lbl.akenti.certs.elements.AttributeInfo;
import lbl.akenti.certs.elements.CertificateAuthorityInfo;
import lbl.akenti.certs.elements.Condition;
import lbl.akenti.certs.elements.Directory;
import lbl.akenti.certs.elements.DistinguishedName;
import lbl.akenti.certs.elements.UseCondIssuerGroup;
import lbl.akenti.util.ArgumentParser;
import lbl.akenti.util.Base64;

/**
 * example: PrintAkentiCertTemplate -R
 * options:
 * [-R]        - Root Policy certificate
 * [-P]        - Policy certificate
 * [-U]        - Use-Condition certificate
 * [-A]        - Attribute certificate
 * [-o]        - output file name
 * [-v]        - verbose
 * [-h]        - prints this message
 *
 * @author Tommaso Sciortino
 */
public class PrintAkentiCertTemplate {
 
  /**
   * This command line program prints a template for a certificate.
   * @param args Command line arguments
   */
  public static void main(String[] args) {
    String outFileName = null;
    String type = null;
    boolean verbose = false;
    
    try{
      ArgumentParser parser = new ArgumentParser(args);
      
      if (args.length == 0) {
        printUsage();
        System.exit(2);
      }
      
      parser.parseArgumentTokens(new char[] {'o','v','R','P','U','A', 'h'} );
      
      outFileName = parser.getOptionArg('o');
      verbose = (parser.getOption('v'));
      if(parser.getOption('R')) {
        type = "RootPolicy";
      } else if(parser.getOption('P')) {
        type = "Policy";
      } else if(parser.getOption('U')) {
        type = "UseCondition";
      } else if(parser.getOption('A')) {
        type = "Attribute";
      }
      
      if (parser.getOption('h')) {
        printUsage();
        System.exit(2);
      } else if (type == null) {
        System.err.println("You must specify what type of certificate you want printed");
        System.exit(2);
      }
      
    } catch (IOException ex) {
      System.err.println(ex.getMessage());
      printUsage();
      System.exit(2);
    }
    
    try{
      
      AkentiCertificate cert = null;
      
      if (type.equals("UseCondition")) {
        UseCondCertGenerator gen = new UseCondCertGenerator();
        gen.setIssuer(new AkentiIssuer());
        String[] actions = {"read","write"};
        gen.setActions(actions);
        DistinguishedName[] dns = { new DistinguishedName() };
        AkentiPrincipal[] aps = { new AkentiPrincipal() };
        Directory[] dirs = { new Directory() };
        AttributeInfo[] ais = { new AttributeInfo("CN","NO_CN",aps,dirs),
        new AttributeInfo("CN","NO_CN",dns),
        new AttributeInfo("CN","NO_CN")};
        gen.setCondition(new Condition("GROUP = NO_GROUP_NAME  || CN = NO_CN && TIME > NO_TIME", ais));
        cert = gen.generate();
      } else if (type.equals("Attribute")) {
        AttributeCertGenerator gen = new AttributeCertGenerator();
        gen.setIssuer(new AkentiIssuer());
        cert = gen.generate();
      } else if (type.equals("Policy")) {
        PolicyCertGenerator gen = new PolicyCertGenerator();
        gen.setIssuer(new AkentiIssuer());
        UseCondIssuerGroup[] ucig = { new UseCondIssuerGroup() };
        gen.setUseCondIssuerGroups(ucig);
        cert = gen.generate();
      } else if (type.equals("RootPolicy")) {
        PolicyCertGenerator gen = new PolicyCertGenerator();
        gen.setIssuer(new AkentiIssuer());
        UseCondIssuerGroup[] ucig = { new UseCondIssuerGroup() };
        CertificateAuthorityInfo x = new CertificateAuthorityInfo(createX509());
        gen.setCertificateAuthorityInfos(new CertificateAuthorityInfo[] {x} );
        gen.setUseCondIssuerGroups(ucig);
        cert = gen.generate();
      } else {
        throw new RuntimeException("illegal certificate type requested");
      }
      
      AkentiCertificateFactory.writeXMLCertToStream(cert,System.out);
      
      System.exit(0);
      
    } catch(RuntimeException ex) {
      System.err.println("SHOULD NEVER HAPPEN");
      ex.printStackTrace();
    } catch(Exception ex) {
      ex.printStackTrace();
    }
  }
  
  /**
   * Used to create a dummy x509 certificate for Root Policy Certificate
   * @throws CertificateException if x509 is not well-formed (should never happen)
   * @return the x509 certificate
   */  
  static X509Certificate[] createX509() throws CertificateException{
    CertificateFactory cfac = CertificateFactory.getInstance("X.509");
    String x = "MIIBEDCBu6ADAgEAAgEAMA0GCSqGSIb3DQEBBAUAMBIxEDAOBgNVBAMTB05PX" +
    "05BTUUwHhcNMDQwNjI5MjIwNDI4WhcNMDQwNjI5MjIwNDI4WjASMRAwDgYDVQQDEwdOT19OQ" +
    "U1FMFowDQYJKoZIhvcNAQEBBQADSQAwRgJBAL8eh2Sj/vl6w9YPVpr6sP/6q8Ufcf5lsJWfx" +
    "Rr2u7ezoHFZBaE2uYyvtz9iSSJ2K0WquLIyYUF9DQ6JGW1DPKMCAQMwDQYJKoZIhvcNAQEEB" +
    "QADQQBRPeuBHdLHlNNumF2j+vj145tpyvfMqWYAbAX7+lXmOwhbfTPEw1LsCVtnS4X5dHEGk" +
    "23qsE9efvI51FDO7vbP";
    ByteArrayInputStream bais = new ByteArrayInputStream(Base64.decode(x));
    X509Certificate x509 = (X509Certificate) cfac.generateCertificate(bais);
    return new X509Certificate[] {x509};
  }
  
  /**
   * Prints the usage instructions for this program
   */  
  static void printUsage() {
    System.err.println("example: PrintAkentiCertTemplate -R");
    System.err.println("options:");
    System.err.println("  [-R]        - Root Policy certificate");
    System.err.println("  [-P]        - Policy certificate");
    System.err.println("  [-U]        - Use-Condition certificate");
    System.err.println("  [-A]        - Attribute certificate");
    System.err.println("  [-o]        - output file name");
    System.err.println("  [-v]        - verbose");
    System.err.println("  [-h]        - prints this message");
  }
}