/*
 * RemoteCheckAccess.java
 *
 * Created on July 21, 2004, 3:00 PM
 */

package lbl.akenti.tools;

import java.io.IOException;
import java.net.URL;
import lbl.akenti.certs.AkentiCertificateFactory;
import lbl.akenti.messaging.AkentiMessage;
import lbl.akenti.messaging.CertificateInfo;
import lbl.akenti.messaging.CheckPolicyRequest;
import lbl.akenti.messaging.CheckPolicyResponse;
import lbl.akenti.messaging.PolicyInfo;
import lbl.akenti.protocolHandler.RemoteProtocolHandler;
import lbl.akenti.util.AkentiConstants;
import lbl.akenti.util.ArgumentParser;
import lbl.akenti.util.ConfFileParser;

/**
 *  example: CheckPolicy -f conf.txt -r "R1"
 * -F arg       - akenti engine config file
 * -r arg       - resource
 * [-l arg]     - level 1, 2, or 3
 * [-a]         - use level 3
 * [-v]         - verbose
 * [-h]         - prints this message
 * where each level is as follows:
 *  1 - prints WORKED/FAILED only
 *  2 - prints all messages collected (this is the default level)
 *  3 - prints level 3 and all the certificates as well, same as -a option
 *
 * @author Tommaso Sciortino
 */
public class RemoteCheckPolicy implements AkentiConstants {
  
  /**
   * Use this program to remotely collect a resource's Policy
   * & UseCondition Certificate.
   * @param args Command line argument.
   */
  public static void main(String[] args) {
    String confFileName = null;
    String resource = null;
    int level = 2;
    
    boolean verbose = false;
    
    try{
      ArgumentParser parser = new ArgumentParser(args);
      
      if (args.length == 0) {
        printUsage();
        System.exit(2);
      }
      
      char[] flags = new char[] {'F', 'r', 'a', 'l', 'v', 'h'};
      parser.parseArgumentTokens(flags);
      
      confFileName = parser.getOptionArg('F');
      resource = parser.getOptionArg('r');
      if (parser.getOptionArg('l') != null) {
        try {
          level = new Integer(parser.getOptionArg('l')).intValue();
        } catch(NumberFormatException ex) {
          System.out.println("could not parse level number");
          System.exit(2);
        }
      } else if (parser.getOption('a')) {
        level = 3;
      }
      
      verbose = parser.getOption('v');
      
      if (parser.getOption('h')) {
        printUsage();
        System.exit(2);
      } else if (confFileName == null || confFileName.equals("")) {
        System.err.println("Must provide a config file");
        System.exit(2);
      } else if (resource == null || resource.equals("")) {
        System.err.println("Must provide a resource");
        System.exit(2);
      } else if (level < 1 || level > 3) {
        System.err.println("level must be between either 1, 2, or 3");
        System.exit(2);
      }
      
    } catch (IOException ex) {
      System.err.println(ex.getMessage());
      printUsage();
      System.exit(2);
    }
    
    try {
      ConfFileParser.loadFromFile(confFileName);
     
      String host = System.getProperty("AkentiHost", AkentiHost);
      String sslMode = System.getProperty("SslMode", SslMode);
      int port = 0;
      URL url;
      
      if (sslMode.equals("on")) {
        SSLUtil.initSSL();
        port = new Integer(System.getProperty("AkentiPort", AkentiPortSSL)).intValue();
        url = new URL("https", host, port,"/");
      } else {
        port = new Integer(System.getProperty("AkentiPort", AkentiPort)).intValue();
        url = new URL("http", host, port,"/");
      }
      
      RemoteProtocolHandler eng = new RemoteProtocolHandler(url);
      CheckPolicyRequest req = new CheckPolicyRequest(RemoteProtocolHandler.getID(),resource);
      CheckPolicyResponse resp = null;
      
      try {
        resp = eng.process(req);
      } catch(Exception ex) {
        System.err.println("There was an error connecting to " + url.toString());
        ex.printStackTrace();
        System.exit(3);
      }
      
      if (verbose) {
        System.out.println(resp.getMessage());
      }
      
      System.out.println("----------------------");
      if (resp.getMessage().getCode() == 150) {
        System.out.println("CheckPolicy WORKED");
      } else {
        System.out.println("CheckPolicy FAILED");
      }
      System.out.println("----------------------");
      
      if (level == 1) {
        if (resp.getMessage().getCode() == 150) {
          System.exit(0);
        } else {
          System.exit(1);
        }
      }
      PolicyInfo[] pis = resp.toPolicyInfosArray();
      
      for (int i=0; i < pis.length; i++) {
        String pisRes = pis[i].getResource();
        CertificateInfo[] cis = pis[i].toCertificateInfosArray();
        for (int j=0; j < cis.length; j++) {
          System.out.println("----------------------");
          String type = null;
          if ( cis[j].getAkentiCertificate() != null) {
            type = cis[j].getAkentiCertificate().getType();
          } else {
            type = "dummy";
          }
          
          System.out.print("Resource = " + pisRes + ", Used = " + cis[j].isUsed());
          if (type.equals("dummy")) {
            System.out.println(", Top Level Error");
          } else {
            System.out.println(", Type = " + type);
          }
          
          if (type.equals("UseCondition")) {
            System.out.println("Location = " + cis[j].getLocation());
          }
          
          AkentiMessage[] ams = cis[j].getAkentiMessages();
          for (int k=0; k < ams.length; k++) {
            System.out.println(ams[k].toString());
          }
          
          if (level == 3 && !type.equals("dummy")) {
            AkentiCertificateFactory.writeCertToStream("XML", cis[j].getAkentiCertificate(), System.out);
          }
          
        }
      }
      
      
      if (resp.getMessage().getCode() < 200) {
        System.exit(0);
      } else {
        System.exit(1);
      }
    } catch(RuntimeException ex) {
      System.err.println("SHOULD NEVER HAPPEN");
      ex.printStackTrace();
      System.exit(3);
    } catch(Exception ex) {
      ex.printStackTrace();
      System.exit(3);
    }
    
  }
  
  /**
   * Prints the usage instructions for this program
   */
  static void printUsage() {
    System.err.println("example: CheckPolicy -f conf.txt -r R1");
    System.err.println("options:");
    System.err.println("  -F arg       - akenti engine config file");
    System.err.println("  -r arg       - resource");
    System.err.println("  [-l arg]     - level 1, 2, or 3");
    System.err.println("  [-a]         - use level 3");
    System.err.println("  [-v]         - verbose");
    System.err.println("  [-h]         - prints this message");
    System.err.println("where each level is as follows:");
    System.err.println(" 1 - prints WORKED/FAILED only");
    System.err.println(" 2 - prints all messages collected (this is the default level)");
    System.err.println(" 3 - prints level 3 and all the certificates as well, same as -a option");
  }
  
}